from __future__ import print_function, division, absolute_import

#
# Copyright (c) 2010 - 2012 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#

"""
Contains classes for working with x.509 certificates.

Several of the classes in this module are now marked deprecated in favor
of their new counterparts in certificate2 module. However, rather than
depending on either specifically, you can use the create methods below to
automatically create the correct object for any given certificate.

Eventually the deprecated classes below will be removed, and the new classes
will be relocated into this module.
"""
import dateutil
import os
import re
from datetime import datetime as dt
from datetime import tzinfo, timedelta
from rhsm import _certificate
import logging
import warnings

log = logging.getLogger(__name__)


# Regex used to scan for OIDs:
OID_PATTERN = re.compile('([0-9]+\.)+[0-9]+:')


# Regex used to parse OID values such as:
#    0:d=0  hl=2 l=   3 prim: UTF8STRING        :2.0
VALUE_PATTERN = re.compile('.*prim:\s(\w*)\s*:*(.*)')


# NOTE: These factory methods create new style certificate objects from
# the certificate2 module. They are placed here to abstract the fact that
# we're using two modules for the time being. Eventually the certificate2 code
# should be moved here.
def create_from_file(path):
    from rhsm.certificate2 import _CertFactory  # prevent circular deps
    return _CertFactory().create_from_file(path)


def create_from_pem(pem):
    from rhsm.certificate2 import _CertFactory  # prevent circular deps
    return _CertFactory().create_from_pem(pem)


def parse_tags(tag_str):
    """
    Split a comma separated list of tags from a certificate into a list.
    """
    tags = []
    if tag_str:
        tags = tag_str.split(",")
    return tags


class UTC(tzinfo):
    """UTC"""

    _ZERO = timedelta(0)

    def utcoffset(self, dt):
        return self._ZERO

    def tzname(self, dt):
        return "UTC"

    def dst(self, dt):
        return self._ZERO


def get_datetime_from_x509(date):
    return dateutil.parser.parse(date)


def deprecated(func):
    """
    A decorator that marks a function as deprecated. This will cause a
    warning to be emitted any time that function is used by a caller.
    """
    def new_func(*args, **kwargs):
        warnings.warn("Call to deprecated function: %s" % func.__name__,
                category=DeprecationWarning)
        return func(*args, **kwargs)
    new_func.__name__ = func.__name__
    new_func.__doc__ = func.__doc__
    new_func.__dict__.update(func.__dict__)
    return new_func


class Certificate(object):
    """
    Represents and x.509 certificate.

    :ivar x509: The :obj:`X509` backing object.
    :type x509: :class:`X509`
    :ivar __ext: A dictionary of extensions `OID`:value
    :type __ext: dict of :obj:`Extensions`
    """

    @deprecated
    def __init__(self, content=None):
        """
        :param content: The (optional) PEM encoded content.
        :type content: str
        """
        self._update(content)

    def _update(self, content):
        if content:
            x509 = _certificate.load(pem=content)
            if x509 is None:
                raise CertificateException("Error loading certificate")
        else:
            x509 = _certificate.X509()
        self.__ext = Extensions(x509)
        self.x509 = x509

        self.subj = self.x509.get_subject()
        self.serial = self.x509.get_serial_number()

        self.altName = x509.get_extension(name='subjectAltName')

    def serialNumber(self):
        """
        Get the serial number

        :return: The x.509 serial number
        :rtype:  str
        """
        return self.serial

    def subject(self):
        """
        Get the certificate subject.

        note: Missing NID mapping for UID added to patch openssl.

        :return: A dictionary of subject fields.
        :rtype: dict
        """
        return self.subj

    def alternateName(self):
        """
        Return the alternate name of the certificate.

        :return: A string representation of the alternate name
        :rtype: str
        """
        return self.altName

    def validRange(self):
        """
        Get the valid date range.

        :return: The valid date range.
        :rtype: :class:`DateRange`
        """
        return DateRange(get_datetime_from_x509(self.x509.get_not_before()),
                get_datetime_from_x509(self.x509.get_not_after()))

    def valid(self, on_date=None):
        """
        Get whether the certificate is valid based on date.

        :return: True if valid.
        :rtype: boolean
        """
        valid_range = self.validRange()
        gmt = dt.utcnow()
        if on_date:
            gmt = on_date
        gmt = gmt.replace(tzinfo=GMT())
        return valid_range.has_date(gmt)

    def expired(self, on_date=None):
        """
        Get whether the certificate is expired based on date.

        :return: True if valid.
        :rtype: boolean
        """
        valid_range = self.validRange()
        gmt = dt.utcnow()
        if on_date:
            gmt = on_date
        gmt = gmt.replace(tzinfo=GMT())
        return valid_range.end() < gmt

    def bogus(self):
        """
        Get whether the certificate contains bogus data or is otherwise unsuitable.

        The certificate may be valid but still be considered bogus.

        :return: List of reasons if bogus
        :rtype: list
        """
        return []

    def extensions(self):
        """
        Get custom extensions.

        :return: An extensions object.
        :rtype: :class:`Extensions`
        """
        return self.__ext

    # TODO: This looks like it should be in the c-tor:
    def read(self, pem_path):
        """
        Read a certificate file.

        :param pem_path: The path to a .pem file.
        :type pem_path: str
        :return: A certificate
        :rtype: :class:`Certificate`
        """
        f = open(pem_path)
        content = f.read()
        try:
            self._update(content)
        finally:
            f.close()
        self.path = pem_path

    def write(self, pem_path):
        """
        Write the certificate.

        :param pem_path: The path to the .pem file.
        :type pem_path: str
        :return: self
        :rtype :class:`Certificate`
        """
        f = open(pem_path, 'w')
        f.write(self.toPEM())
        self.path = pem_path
        f.close()
        return self

    def delete(self):
        """
        Delete the file associated with this certificate.
        """
        if hasattr(self, 'path'):
            os.unlink(self.path)
        else:
            raise Exception('no path, not deleted')

    def toPEM(self):
        """
        Get PEM representation of the certificate.

        :return: A PEM string
        :rtype: str
        """
        return self.x509.as_pem()

    def __str__(self):
        return self.x509.as_text()

    def __repr__(self):
        sn = self.serialNumber()
        cert_path = None
        if hasattr(self, 'path'):
            cert_path = self.path
        return '[sn: %d, path: "%s"]' % (sn, cert_path)

    def __cmp__(self, other):
        valid_range = self.validRange()
        exp1 = valid_range.end()
        other_valid_range = other.validRange()
        exp2 = other_valid_range.end()
        if exp1 < exp2:
            return -1
        if exp1 > exp2:
            return 1
        return 0


class RedhatCertificate(Certificate):
    """
    Represents a Red Hat certificate.

    :cvar REDHAT: The Red Hat base OID.
    :type REDHAT: str
    """

    REDHAT = '1.3.6.1.4.1.2312.9'

    def __init__(self, *args, **kwargs):
        super(RedhatCertificate, self).__init__(*args, **kwargs)
        self._extract_redhat_extensions()

    def _extract_redhat_extensions(self):
        self.__redhat = self.extensions().branch(self.REDHAT)

    def _update(self, content):
        Certificate._update(self, content)
        self._extract_redhat_extensions()

    def redhat(self):
        """
        Get the extension subtree for the `redhat` namespace.

        :return: The extensions with the Red Hat namespace trimmed.
        :rtype: :class:`Extensions`
        """
        return self.__redhat

    def bogus(self):
        bogus = Certificate.bogus(self)
        if self.serialNumber() < 1:
            bogus.append('Serial number must be > 0')
        cn = self.subject().get('CN')
        if not cn:
            bogus.append('Invalid common name: %s' % cn)
        return bogus


class ProductCertificate(RedhatCertificate):
    """
    Represents a Red Hat product/entitlement certificate.

    It is OID schema aware and provides methods to
    get product information.
    """

    def getProduct(self):
        """
        Get the product defined in the certificate.

        :return: A product object.
        :rtype: :class:`Product`
        """
        rhns = self.redhat()
        products = rhns.find('1.*.1', 1)
        if products:
            p = products[0]
            oid = p[0]
            root = oid.rtrim(1)
            product_id = oid[1]
            ext = rhns.branch(root)
            return Product(product_id, ext)

    def getProducts(self):
        """
        Get a list products defined in the certificate.

        :return: A list of product objects.
        :rtype: list of :class:`Product`
        """
        lst = []
        rhns = self.redhat()
        for p in rhns.find('1.*.1'):
            oid = p[0]
            root = oid.rtrim(1)
            product_id = oid[1]
            ext = rhns.branch(root)
            lst.append(Product(product_id, ext))
        return lst

    def bogus(self):
        bogus = RedhatCertificate.bogus(self)
        return bogus

    def __str__(self):
        s = []
        s.append('RAW:')
        s.append('===================================')
        s.append(Certificate.__str__(self))
        s.append('MODEL:')
        s.append('===================================')
        s.append('Serial#: %s' % self.serialNumber())
        s.append('Subject (CN): %s' % self.subject().get('CN'))
        for p in self.getProducts():
            s.append(str(p))
        return '\n'.join(s)


class EntitlementCertificate(ProductCertificate):
    """
    Represents an entitlement certificate.
    """
    def _update(self, content):
        ProductCertificate._update(self, content)

        rhns = self.redhat()
        order = rhns.find('4.1', 1, True)
        if order:
            p = order[0]
            oid = p[0]
            root = oid.rtrim(1)
            ext = rhns.branch(root)
            self.order = Order(ext)
        else:
            self.order = None

    def delete(self):
        """
        Delete the file associated with this certificate.
        """
        if hasattr(self, 'path'):
            os.unlink(self.path)
            # we should keep the key path around, but
            # we dont seem to, but it's consistent
            parts = self.path.split('.')
            key_path = "%s-key.pem" % parts[0]
            os.unlink(key_path)
        else:
            raise Exception('no path, not deleted')

    def getOrder(self):
        """
        Get the :obj:`order` object defined in the certificate.

        :return: An order object.
        :rtype: :class:`Order`
        """
        return self.order

    def getEntitlements(self):
        """
        Get all entitlements defined in the certificate.

        :return: A list of entitlement object.
        :rtype: List of :class:`Entitlement`
        """
        return self.getContentEntitlements() \
             + self.getRoleEntitlements()

    # TODO: Not a great name, this is just getting content, self is
    # the entitlement.
    def getContentEntitlements(self):
        """
        Get the B{content} entitlements defined in the certificate.

        :return: A list of entitlement object.
        :rtype: [:obj:`Content`,..]
        """
        lst = []
        rhns = self.redhat()
        entitlements = rhns.find('2.*.1.1')
        for ent in entitlements:
            oid = ent[0]
            root = oid.rtrim(1)
            ext = rhns.branch(root)
            lst.append(Content(ext))
        return lst

    def getRoleEntitlements(self):
        """
        Get the *role* entitlements defined in the certificate.

        :return: A list of entitlement object.
        :rtype: [:obj:`Role`,..]
        """
        lst = []
        rhns = self.redhat()
        entitlements = rhns.find('3.*.1')
        for ent in entitlements:
            oid = ent[0]
            root = oid.rtrim(1)
            ext = rhns.branch(root)
            lst.append(Role(ext))
        return lst

    @deprecated
    def validRangeWithGracePeriod(self):
        return super(EntitlementCertificate, self).validRange()

    @deprecated
    def validWithGracePeriod(self):
        return self.validRangeWithGracePeriod().has_now()

    def bogus(self):
        bogus = ProductCertificate.bogus(self)
        if self.getOrder() is None:
            bogus.append('No order infomation')
        return bogus

    def __str__(self):
        s = []
        order = self.getOrder()
        s.append(ProductCertificate.__str__(self))
        for ent in self.getEntitlements():
            s.append(str(ent))
        s.append(str(order))
        return '\n'.join(s)


class Key(object):
    """
    The (private|public) key.

    :ivar content: The PEM encoded key.
    :type content: str
    """

    @classmethod
    def read(cls, pem_path):
        """
        Read the key.

        :param pem_path: The path to the .pem file.
        :type path: str
        """
        f = open(pem_path)
        content = f.read()
        f.close()
        key = Key(content)
        key.path = pem_path
        return key

    def __init__(self, content):
        """
        :param content: The PEM encoded key.
        :type content: str
        """
        self.content = content

    def bogus(self):
        bogus = []
        if self.content:
            # TODO handle public keys too? class docstring seems to indicate so
            key = _certificate.load_private_key(pem=self.content)
            if not key:
                bogus.append("Invalid key data")
        else:
            bogus.append("No key data provided")

        return bogus

    def write(self, pem_path):
        """
        Write the key.

        :param path: The path to the .pem file.
        :type path: str
        :return: self
        """
        f = open(pem_path, 'w')
        f.write(self.content)
        self.path = pem_path
        f.close()
        os.chmod(pem_path, 0o644)
        return self

    def delete(self):
        """
        Delete the file associated with this key.
        """
        if hasattr(self, 'path'):
            os.unlink(self.path)
        else:
            raise Exception('no path, not deleted')

    def __str__(self):
        return self.content


class DateRange(object):
    """
    Date range object.

    :ivar begin_date: The begining date
    :type begin_date: :class:`datetime`
    :ivar end_date: The ending date
    :type end_date: :class:`datetime`
    """

    def __init__(self, begin_date, end_date):
        self._begin = self._convert_to_utc(begin_date)
        self._end = self._convert_to_utc(end_date)

    def _convert_to_utc(self, timestamp):
        if timestamp.tzinfo is None:
            return timestamp.replace(tzinfo=GMT())
        else:
            return timestamp.astimezone(GMT())

    def begin(self):
        """
        Get range beginning.

        :return: The beginning date in UTC.
        :rtype: :class:`datatime.datetime`
        """
        return self._begin

    def end(self):
        """
        Get range end.

        :return: The end date in UTC.
        :rtype: :class:`datetime.datetime`
        """
        return self._end

    def has_now(self):
        """
        Get whether the certificate is valid based on the date now.

        :return: True if valid.
        :rtype: boolean
        """
        gmt = dt.utcnow()
        gmt = gmt.replace(tzinfo=GMT())
        return self.has_date(gmt)

    def has_date(self, date):
        """
        Get whether the certificate is valid based on the date.

        :param: date
        :type: :class:`datetime.datetime`
        :return: True if valid.
        :rtype: boolean
        """
        return (date >= self.begin() and date <= self.end())

    @deprecated
    def hasDate(self, date):
        return self.has_date(date)

    @deprecated
    def hasNow(self):
        return self.has_now()

    def __str__(self):
        return '\n\t%s\n\t%s' % (self._begin, self._end)


class GMT(tzinfo):
    """GMT"""

    def utcoffset(self, date_time):
        return timedelta(seconds=0)

    def tzname(self, date_time):
        return 'GMT'

    def dst(self, date_time):
        return timedelta(seconds=0)


class Extensions(dict):
    """
    Represents x.509 (v3) custom extensions.
    """

    def __init__(self, x509):
        """
        :param x509: An :module:`rhsm._certificate` :class:`X509` object or dict.
        :type x509: :obj:`X509`
        """
        if isinstance(x509, dict):
            self.update(x509)
        else:
            self._parse(x509)

    def ltrim(self, n):
        """
        Left trim *n* parts.

        :param n: The number of parts to trim.
        :type n: int
        :return: The trimmed OID
        :rtype: :class:`Extensions`
        """
        d = {}
        for oid, v in list(self.items()):
            d[oid.ltrim(n)] = v
        return Extensions(d)

    def get(self, oid, default=None):
        """
        Get the value of an extension by *oid*.

        Note: The *oid* may contain (*) wildcards.

        :param oid: An `OID` that may contain (*) wildcards.
        :type oid: str|`OID`
        :return: The value of the first extension matched.
        :rtype: str
        """
        ext = self.find(oid, 1, True)
        if ext:
            return ext[0][1]
        else:
            return default

    def find(self, oid, limit=0, ignoreOrder=False):
        """
        Find all extensions matching the oid.

        Note: The oid may contain (*) wildcards.

        :param oid: An OID that may contain (*) wildcards.
        :type oid: str|`OID`
        :param limit: Limit the number returned, 0=unlimited
        :type limit: int
        :param ignoreOrder Should oids be ordered
        :type ignoreOrder: bool
        :return: A list of matching items.
        :rtype: (`OID`, value)
        :see: OID.match()
        """
        ext = []
        found = 0
        if isinstance(oid, str):
            oid = OID(oid)

        # Only order the keys if we want more than a singel return avalue
        if ignoreOrder:
            keyset = list(self.keys())
        else:
            keyset = sorted(self.keys())

        for k in keyset:
            if k.match(oid):
                v = self[k]
                ext.append((k, v))
                found = found + 1
            if limit and found == limit:
                break
        return ext

    def branch(self, root):
        """
        Find a subtree by matching the oid.

        :param root: An `OID` that may contain (*) wildcards.
        :type root: str|`OID`
        :return: A subtree.
        :rtype: :class:`Extensions`
        """
        d = {}
        if isinstance(root, str):
            root = OID(root)
        if root[-1]:
            root = root.append('')
        ln = len(root) - 1
        for oid, v in self.find(root):
            trimmed = oid.ltrim(ln)
            d[trimmed] = v
        return Extensions(d)

    def _parse(self, x509):
        """
        Parse the extensions. Expects an :module:`rhsm._certificate` :class:`X509` object.
        """
        for oid, value in list(x509.get_all_extensions().items()):
            oid = OID(oid)
            self[oid] = value

    def __str__(self):
        s = []
        for item in list(self.items()):
            s.append('%s = "%s"' % item)
        return '\n'.join(s)


class OID(object):
    """
    The Object Identifier object.

    :ivar part: The oid parts.
    :type part: [str,]
    :cvar WILDCARD: The wildcard character.
    :type WILDCARD: str
    """

    WILDCARD = '*'

    @classmethod
    def join(cls, *oid):
        return '.'.join(oid)

    @classmethod
    def split(cls, s):
        """
        Split an OID string.

        :param s: An OID string Eg: (1.2.3)
        :type s: str
        :return: A list of OID parts.
        :rtype: [str,]
        """
        return s.split('.')

    def __init__(self, oid):
        """
        :param oid: The OID value.
        :type oid: str|[str,]
        """
        if isinstance(oid, str):
            self.part = self.split(oid)
        else:
            self.part = oid

        self._len = None
        self._str = None
        self._hash = None

    def parent(self):
        """
        Get the parent OID.

        :return: The parent OID.
        :rtype: L{OID}
        """
        p = self.part[:-1]
        if p:
            return OID(p)

    def ltrim(self, n):
        """
        Left trim I{n} parts.

        :param n: The number of parts to trim.
        :type n: int
        :return: The trimmed OID
        :rtype: :class:`OID`
        """
        return OID(self.part[n:])

    def rtrim(self, n):
        """
        Right trim I{n} parts.

        :param n: The number of parts to trim.
        :type n: int
        :return: The trimmed OID
        :rtype: :class:`OID`
        """
        return OID(self.part[:-n])

    def append(self, oid):
        """
        Append the specified OID fragment.

        :param oid: An OID fragment.
        :type oid: str|`OID`
        :return: The concatenated OID.
        "rtype: :class:`OID`
        """
        if isinstance(oid, str):
            oid = OID(oid)
        part = self.part + oid.part
        return OID(part)

    def match(self, oid):
        """
        Match the specified OID considering wildcards.

        Patterns:
          - 1.4.5.6.74 (not wildcarded)
          -    .5.6.74 (match on only last 4)
          -    5.6.74. (match only first 4)
          - 1.4.*.6.*  (wildcard pattern)

        :param oid: An OID string or object.
        :type oid: `OID`
        :return: True if matched
        :rtype: boolean
        """
        i = 0

        # Matching the end
        if not oid[0]:
            #oid = OID(oid[1:])
            oid = oid[1:]
            parts = self.part[-len(oid):]
        # Matching the beginning
        elif not oid[-1]:
            #oid = OID(oid[:-1])
            oid = oid[:-1]
            parts = self.part[:len(oid)]
        # Full on match
        else:
            parts = self.part

        # The lengths do not match, fail.
        if len(parts) != len(oid):
            return False

        for x in parts:
            val = oid[i]
            if (x == val or val == self.WILDCARD):
                i += 1
            else:
                return False

        return True

    def __len__(self):
        if not self._len:
            self._len = len(self.part)

        return self._len

    def __getitem__(self, index):
        return self.part[index]

    def __repr__(self):
        return str(self)

    def __hash__(self):
        if not self._hash:
            self._hash = hash(str(self))

        return self._hash

    def __eq__(self, other):
        return (str(self) == str(other))

    def __lt__(self, other):
        return str(self) < str(other)

    def __str__(self):
        if not self._str:
            self._str = '.'.join(self.part)

        return self._str


class Order(object):

    @deprecated
    def __init__(self, ext):
        self.ext = ext

    def getName(self):
        return self.ext.get('1')

    def getNumber(self):
        return self.ext.get('2')

    def getSku(self):
        return self.ext.get('3')

    def getSubscription(self):
        return self.ext.get('4')

    def getQuantity(self):
        return self.ext.get('5')

    def getStart(self):
        return self.ext.get('6')

    def getEnd(self):
        return self.ext.get('7')

    def getVirtLimit(self):
        return self.ext.get('8')

    def getSocketLimit(self):
        return self.ext.get('9')

    def getContract(self):
        return self.ext.get('10')

    def getQuantityUsed(self):
        """
        Returns the quantity of the subscription that *this* entitlement is using.

        WARNING: a little misleading as it (a) is part of the order namespace
        and (b) sounds suspiciously like the total consumed quantity of the
        subscription.
        """
        return self.ext.get('11')

    def getWarningPeriod(self):
        return self.ext.get('12')

    def getAccountNumber(self):
        return self.ext.get('13')

    def getProvidesManagement(self):
        return self.ext.get('14')

    def getSupportLevel(self):
        return self.ext.get('15')

    def getSupportType(self):
        return self.ext.get('16')

    def getStackingId(self):
        return self.ext.get('17')

    def getVirtOnly(self):
        return self.ext.get('18')

    def __str__(self):
        s = []
        s.append('Order {')
        s.append('\tName .............. = %s' % self.getName())
        s.append('\tNumber ............ = %s' % self.getNumber())
        s.append('\tSKU ............... = %s' % self.getSku())
        s.append('\tSubscription ...... = %s' % self.getSubscription())
        s.append('\tQuantity .......... = %s' % self.getQuantity())
        s.append('\tStart (Ent) ....... = %s' % self.getStart())
        s.append('\tEnd (Ent) ......... = %s' % self.getEnd())
        s.append('\tVirt Limit ........ = %s' % self.getVirtLimit())
        s.append('\tSocket Limit ...... = %s' % self.getSocketLimit())
        s.append('\tContract .......... = %s' % self.getContract())
        s.append('\tWarning Period .... = %s' % self.getWarningPeriod())
        s.append('\tAccount Number .... = %s' % self.getAccountNumber())
        s.append('\tProvides Management = %s' % self.getProvidesManagement())
        s.append('\tSupport Level ..... = %s' % self.getSupportLevel())
        s.append('\tSupport Type ...... = %s' % self.getSupportType())
        s.append('\tStacking Id ....... = %s' % self.getStackingId())
        s.append('\tVirt Only ......... = %s' % self.getVirtOnly())
        s.append('}')
        return '\n'.join(s)


class Product(object):

    @deprecated
    def __init__(self, p_hash, ext):
        self.hash = p_hash
        self.ext = ext
        self.name = self.ext.get('1')
        self.version = self.ext.get('2')
        self.arch = self.ext.get('3')
        self.provided_tags = parse_tags(self.ext.get('4'))
        self.brand_type = self.ext.get('5')
        self.brand_name = self.ext.get('6')

    def getHash(self):
        return self.hash

    def getName(self):
        return self.name

    def getArch(self):
        return self.arch

    def getVersion(self):
        return self.version

    def getProvidedTags(self):
        return self.provided_tags

    def getBrandType(self):
        return self.brand_type

    def getBrandName(self):
        return self.brand_name

    def __eq__(self, rhs):
        return (self.getHash() == rhs.getHash())

    def __str__(self):
        s = []
        s.append('Product {')
        s.append('\tHash ......... = %s' % self.getHash())
        s.append('\tName ......... = %s' % self.getName())
        s.append('\tVersion ...... = %s' % self.getVersion())
        s.append('\tArchitecture . = %s' % self.getArch())
        s.append('\tProvided Tags  = %s' % self.getProvidedTags())
        s.append('\tBrand Type     = %s' % self.getBrandType())
        s.append('\tBrand Name     = %s' % self.getBrandName())
        s.append('}')
        return '\n'.join(s)

    def __repr__(self):
        return str(self)


class Entitlement(object):

    def __init__(self, ext):
        self.ext = ext


class Content(Entitlement):

    def __init__(self, ext):
        Entitlement.__init__(self, ext)
        self.name = self.ext.get('1')
        self.label = self.ext.get('2')
        self.quantity = self.ext.get('3')
        self.flex_quantity = self.ext.get('4')
        self.vendor = self.ext.get('5')
        self.url = self.ext.get('6')
        self.gpg = self.ext.get('7')
        self.enabled = self.ext.get('8')
        self.metadata_expire = self.ext.get('9')
        self.required_tags = parse_tags(self.ext.get('10'))

    def getName(self):
        return self.name

    def getLabel(self):
        return self.label

    def getQuantity(self):
        return self.quantity

    def getFlexQuantity(self):
        return self.flex_quantity

    def getVendor(self):
        return self.vendor

    def getUrl(self):
        return self.url

    def getGpg(self):
        return self.gpg

    def getEnabled(self):
        return self.enabled

    def getMetadataExpire(self):
        return self.metadata_expire

    def getRequiredTags(self):
        return self.required_tags

    def __eq__(self, rhs):
        return isinstance(rhs, self.__class__) and (self.label == rhs.label)

    def __str__(self):
        s = []
        s.append('Entitlement (content) {')
        s.append('\tName .......... = %s' % self.getName())
        s.append('\tLabel ......... = %s' % self.getLabel())
        s.append('\tQuantity ...... = %s' % self.getQuantity())
        s.append('\tFlex Quantity . = %s' % self.getFlexQuantity())
        s.append('\tVendor ........ = %s' % self.getVendor())
        s.append('\tURL ........... = %s' % self.getUrl())
        s.append('\tGPG Key ....... = %s' % self.getGpg())
        s.append('\tEnabled ....... = %s' % self.getEnabled())
        s.append('\tMetadata Expire = %s' % self.getMetadataExpire())
        s.append('\tRequired Tags . = %s' % self.getRequiredTags())
        s.append('}')
        return '\n'.join(s)

    def __repr__(self):
        return str(self)

    def __hash__(self):
        return hash(self.label)


class Role(Entitlement):

    def getName(self):
        return self.ext.get('1')

    def getDescription(self):
        return self.ext.get('2')

    def __eq__(self, rhs):
        return (self.getName() == rhs.getName())

    def __str__(self):
        s = []
        s.append('Entitlement (role) {')
        s.append('\tName ......... = %s' % self.getName())
        s.append('\tDescription .. = %s' % self.getDescription())
        s.append('}')
        return '\n'.join(s)

    def __repr__(self):
        return str(self)


class CertificateException(Exception):
    pass



from __future__ import print_function, division, absolute_import

#
# Copyright (c) 2013 Red Hat, Inc.
#
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#
# module for updating product branding info
# on subscription


import logging

log = logging.getLogger(__name__)


class BrandsInstaller(object):
    def __init__(self, ent_certs=None):
        self.ent_certs = ent_certs

        # find brand installers
        self.brand_installers = self._get_brand_installers()

    def _get_brand_installers(self):
        """returns a list or iterable of BrandInstaller(s)"""
        return []

    def install(self):
        for brand_installer in self.brand_installers:
            brand_installer.install()


class BrandInstaller(object):
    """Install branding info for a set of entititlement certs."""

    def __init__(self, ent_certs=None):
        self.ent_certs = ent_certs

        log.debug("BrandInstaller ent_certs:  %s" % [x.serial for x in ent_certs or []])

    def install(self):
        """Create a Brand object if needed, and save it."""

        brand_picker = self._get_brand_picker()
        new_brand = brand_picker.get_brand()

        # no branded name info to install
        if not new_brand:
            return

        current_brand = self._get_current_brand()

        log.debug("Current branded name info, if any: %s" % current_brand.name)
        log.debug("Fresh ent cert has branded product info: %s" % new_brand.name)

        if current_brand.is_outdated_by(new_brand):
            self._install(new_brand)
        else:
            log.debug("Product branding info does not need to be updated")

    def _get_brand_picker(self):
        raise NotImplementedError

    def _get_current_brand(self):
        raise NotImplementedError

    def _install(self, brand):
        raise NotImplementedError


class BrandPicker(object):
    """Returns the branded name to install.

    Check installed product certs, and the list of entitlement certs
    passed in, and find the correct branded name, if any."""

    def __init__(self, ent_certs=None):
        self.ent_certs = ent_certs

    def get_brand(self):
        raise NotImplementedError


class Brand(object):
    """Base class for Brand objects."""

    name = None

    # could potentially be a __lt__ etc, though there is some
    # oddness in the compares are not symetric for the empty
    # cases (ie, we update nothing with something,etc)
    def is_outdated_by(self, new_brand):
        """If a Brand should be replaced with new_brand."""
        if not self.name:
            return True

        # prevent empty branded_name
        if not new_brand.name:
            return False

        # Don't install new branded_name if it's the same to prevent
        # churn
        return new_brand.name != self.name


class ProductBrand(Brand):
    """A brand for a branded product"""
    def __init__(self, name):
        self.brand_file = self._get_brand_file()
        self.name = name

    def _get_brand_file(self):
        return BrandFile()

    def save(self):
        brand = self.format_brand(self.name)
        self.brand_file.write(brand)

    @classmethod
    def from_product(cls, product):
        return cls(product.brand_name)

    @staticmethod
    def format_brand(brand):
        if not brand.endswith('\n'):
            brand += '\n'

        return brand


class CurrentBrand(Brand):
    """The currently installed brand"""
    def __init__(self):
        self.brand_file = self._get_brand_file()
        self.load()

    def _get_brand_file(self):
        return BrandFile()

    def load(self):
        try:
            brand_info = self.brand_file.read()
        except IOError:
            log.error("No brand info file found (%s) " % self.brand_file)
            return

        self.name = self.unformat_brand(brand_info)

    @staticmethod
    def unformat_brand(brand):
        if brand:
            return brand.strip()
        return None


class BrandFile(object):
    """The file used for storing product branding info.

    Default is "/var/lib/rhsm/branded_name
    """

    path = "/var/lib/rhsm/branded_name"

    def write(self, brand_info):
        with open(self.path, 'w') as brand_file:
            brand_file.write(brand_info)

    def read(self):
        with open(self.path, 'r') as brand_file:
            return brand_file.read()

    def __str__(self):
        return "<BrandFile path=%s>" % self.path



from __future__ import print_function, division, absolute_import

#
# Copyright (c) 2012 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#
import datetime
import os
import subprocess

from subscription_manager.ga import Gtk as ga_Gtk

from subscription_manager.gui.utils import get_running_as_firstboot
from subscription_manager.utils import get_client_versions, get_server_versions

from subscription_manager.i18n import ugettext as _

LICENSE = _("\nThis software is licensed to you under the GNU General Public License, "
            "version 2 (GPLv2). There is NO WARRANTY for this software, express or "
            "implied, including the implied warranties of MERCHANTABILITY or FITNESS "
            "FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2 "
            "along with this software; if not, see:\n\n"
            "http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt\n\n"
            "Red Hat trademarks are not licensed under GPLv2. No permission is "
            "granted to use or replicate Red Hat trademarks that are incorporated "
            "in this software or its documentation.\n")

AUTO_ATTACH_UPDATE_FILE = '/run/rhsm/next_auto_attach_update'
CERT_CHECK_UPDATE_FILE = '/run/rhsm/next_cert_check_update'

prefix = os.path.dirname(__file__)


class AboutDialog(object):
    def __init__(self, parent, backend):
        self.backend = backend
        self.dialog = ga_Gtk.AboutDialog()
        self.dialog.set_transient_for(parent)
        self.dialog.set_modal(True)
        self.dialog.set_name(_("Subscription Manager"))
        self.dialog.set_license(LICENSE)
        self.dialog.set_wrap_license(True)
        if not get_running_as_firstboot():
            self.dialog.set_website("https://fedorahosted.org/subscription-manager/")
        self.dialog.set_copyright(_("Copyright (c) 2012 Red Hat, Inc."))
        self.dialog.set_logo_icon_name("subscription-manager")
        self.dialog.set_icon_name("subscription-manager")
        self.dialog.set_authors(["The Subscription Manager Team"])

        next_update_label = ga_Gtk.Label()
        next_auto_attach_label = ga_Gtk.Label()
        sub_man_version_label = ga_Gtk.Label()
        backend_version_label = ga_Gtk.Label()
        context_box = self.dialog.vbox.get_children()[0]
        context_box.pack_end(next_auto_attach_label, True, True, 0)
        context_box.pack_end(next_update_label, True, True, 0)
        context_box.pack_end(sub_man_version_label, True, True, 0)
        context_box.pack_end(backend_version_label, True, True, 0)

        self._set_next_update(next_update_label, next_auto_attach_label)

        # Set the component versions.
        server_versions = get_server_versions(self.backend.cp_provider.get_consumer_auth_cp())
        client_versions = get_client_versions()

        sub_man_version_label.set_markup(_("<b>%s version:</b> %s") %
                                        ("subscription manager", client_versions['subscription-manager']))
        backend_version_label.set_markup(_("<b>subscription management service version:</b> %s") %
                                           server_versions['candlepin'])

        self.dialog.connect("response", self._handle_response)
        self.dialog.show_all()

    def show(self):
        self.dialog.show()

    def _handle_response(self, dialog, response):
        if response == ga_Gtk.ResponseType.DELETE_EVENT or response == ga_Gtk.ResponseType.CANCEL:
            self.dialog.destroy()

    def _set_next_update(self, next_update_label, next_auto_attach_label):
        try:
            if self._rhsmcertd_on():
                next_update = int(open(CERT_CHECK_UPDATE_FILE, 'r').read())
                next_auto_attach = int(open(AUTO_ATTACH_UPDATE_FILE, 'r').read())
            else:
                next_update = None
                next_auto_attach = None
        except Exception:
            next_update = None
            next_auto_attach = None

        if next_update:
            update_time = datetime.datetime.fromtimestamp(next_update)
            next_update_label.set_markup(_('<b>Next System Certificate Check:</b> %s') %
                update_time.strftime("%c"))
            next_update_label.show()
        else:
            next_update_label.hide()
        if next_auto_attach:
            next_auto_attach = datetime.datetime.fromtimestamp(next_auto_attach)
            next_auto_attach_label.set_markup(_('<b>Next System Auto Attach:</b> %s') %
                next_auto_attach.strftime("%c"))
            next_auto_attach_label.show()
        else:
            next_auto_attach_label.hide()

    def _rhsmcertd_on(self):
        fnull = open(os.devnull, "w")
        try:
            # if status == 0 then true

            return not subprocess.call(['pidof', 'rhsmcertd'],
                                       stdout=fnull,
                                       stderr=fnull)
        finally:
            fnull.close()



from __future__ import print_function, division, absolute_import

#
# Copyright (c) 2010 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
try:
    import unittest2 as unittest
except ImportError:
    import unittest

import six

from mock import patch
from mock import Mock
from mock import mock_open

import test.fixture
from test.fixture import OPEN_FUNCTION
from rhsmlib.facts import hwprobe

PROC_BONDING_RR = """Ethernet Channel Bonding Driver: v3.6.0 (September 26, 2009)

Bonding Mode: load balancing (round-robin)
MII Status: up
MII Polling Interval (ms): 100
Up Delay (ms): 0
Down Delay (ms): 0

Slave Interface: eth0
MII Status: up
Speed: 100 Mbps
Duplex: full
Link Failure Count: 0
Permanent HW addr: 52:54:00:07:03:ba
Slave queue ID: 0

Slave Interface: eth1
MII Status: up
Speed: 100 Mbps
Duplex: full
Link Failure Count: 0
Permanent HW addr: 52:54:00:66:20:f7
Slave queue ID: 0
"""

PROC_BONDING_ALB = """Ethernet Channel Bonding Driver: v3.6.0 (September 26, 2009)

Bonding Mode: adaptive load balancing
Primary Slave: None
Currently Active Slave: eth0
MII Status: up
MII Polling Interval (ms): 100
Up Delay (ms): 0
Down Delay (ms): 0

Slave Interface: eth0
MII Status: up
Speed: 100 Mbps
Duplex: full
Link Failure Count: 0
Permanent HW addr: 52:54:00:07:03:ba
Slave queue ID: 0

Slave Interface: eth1
MII Status: up
Speed: 100 Mbps
Duplex: full
Link Failure Count: 0
Permanent HW addr: 52:54:00:66:20:f7
Slave queue ID: 0
"""

PROC_STAT = """cpu  1251631 1521 234099 26677635 43020 53378 18245 0 136791 0
cpu0 170907 672 36975 5235552 21053 18369 7169 0 8215 0
cpu1 162918 87 22790 3049644 2898 7180 2735 0 37398 0
cpu2 167547 104 36469 3045029 6222 6375 2022 0 5443 0
cpu3 116384 86 21325 3103226 2006 3503 999 0 3330 0
cpu4 214747 243 36747 3002774 3602 6008 1688 0 51985 0
cpu5 116967 88 20921 3104477 1791 3092 875 0 4196 0
cpu6 182738 158 37153 3035995 3449 6016 1926 0 19461 0
cpu7 119420 79 21716 3100934 1997 2833 828 0 6759 0
intr 69987280 31 176 0 0 0 0 0 0 1 11832 0 0 723 0 0 0 67 0 26095 0 0 0 0 77 0 0 733270 16903338 0 305 2485413 22 35708 147 618373 6713 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0
ctxt 176256990
btime 1524062304
processes 66111
procs_running 2
procs_blocked 1
softirq 40290891 49 19260365 9256 986071 674800 7 521934 11078954 0 7759455
"""

OS_RELEASE = """NAME="Awesome OS"
VERSION="42 (Go4It)"
ID="awesomeos"
VERSION_ID="42"
PRETTY_NAME="Awesome OS 42 (Go4It)"
CPE_NAME="cpe:/o:awesomeos:best_linux:42:beta:server"

REDHAT_BUGZILLA_PRODUCT="AwesomeOS Enterprise 42"
REDHAT_BUGZILLA_PRODUCT_VERSION=42.0
REDHAT_SUPPORT_PRODUCT="AwesomeOS Enterprise"
REDHAT_SUPPORT_PRODUCT_VERSION=42.0"""

OS_RELEASE_COLON = """NAME="Awesome OS"
VERSION="42 (Go4It)"
ID="awesomeos"
VERSION_ID="42"
PRETTY_NAME="Awesome OS 42 (Go4It)"
CPE_NAME="cpe:/o:awesomeos:best_linux:42:be\:ta:server"

REDHAT_BUGZILLA_PRODUCT="AwesomeOS Enterprise 42"
REDHAT_BUGZILLA_PRODUCT_VERSION=42.0
REDHAT_SUPPORT_PRODUCT="AwesomeOS Enterprise"
REDHAT_SUPPORT_PRODUCT_VERSION=42.0"""


class TestParseRange(unittest.TestCase):
    def test_single(self):
        r = '1'
        r_list = hwprobe.parse_range(r)
        self.assertEqual([1], r_list)

    def test_range_1_4(self):
        r = '1-4'
        r_list = hwprobe.parse_range(r)
        self.assertEqual([1, 2, 3, 4], r_list)


class TestGatherEntries(unittest.TestCase):
    def test_single(self):
        ent = "1"
        ent_list = hwprobe.gather_entries(ent)
        self.assertEqual(1, len(ent_list))

    def test_multiple(self):
        ent = "1,2,3,4"
        ent_list = hwprobe.gather_entries(ent)
        self.assertEqual(4, len(ent_list))

    def test_range_1_2(self):
        ent = "1-2"
        ent_list = hwprobe.gather_entries(ent)
        self.assertEqual(2, len(ent_list))

    def test_range_2_ranges(self):
        ent = "1-4,9-12"
        ent_list = hwprobe.gather_entries(ent)
        self.assertEqual(8, len(ent_list))

    def test_range_64cpu_example(self):
        ent = "0,4,8,12,16,20,24,28,32,36,40,44,48,52,56,60"
        ent_list = hwprobe.gather_entries(ent)
        self.assertEqual(16, len(ent_list))

    def test_range_0_2(self):
        ent = "0,2"
        ent_list = hwprobe.gather_entries(ent)
        self.assertEqual(2, len(ent_list))


class GenericPlatformSpecificInfoProviderTest(test.fixture.SubManFixture):
    def test(self):
        hw_info = {}
        platform_info = hwprobe.GenericPlatformSpecificInfoProvider(hw_info)
        self.assertEqual(0, len(platform_info.info))

    def test_does_nothing(self):
        hw_info = {'foo': '1'}
        platform_info = hwprobe.GenericPlatformSpecificInfoProvider(hw_info)
        self.assertEqual(0, len(platform_info.info))
        self.assertFalse('foo' in platform_info.info)


class HardwareProbeTest(test.fixture.SubManFixture):
    def setUp(self):
        # Note this is patching an *instance* of HardwareCollector, not the class.
        self.hw_check_topo = hwprobe.HardwareCollector()
        self.hw_check_topo_patcher = patch.object(
            self.hw_check_topo,
            'check_for_cpu_topo',
            Mock(return_value=True)
        )
        self.hw_check_topo_patcher.start()
        super(HardwareProbeTest, self).setUp()

    def tearDown(self):
        self.hw_check_topo_patcher.stop()
        super(HardwareProbeTest, self).tearDown()

    @patch(OPEN_FUNCTION)
    def test_distro_no_release(self, MockOpen):
        hw = hwprobe.HardwareCollector()
        MockOpen.side_effect = IOError()
        self.assertRaises(IOError, hw.get_release_info)

    @patch("os.path.exists")
    @patch(OPEN_FUNCTION)
    def test_distro_bogus_content_no_platform_module(self, MockOpen, MockExists):
        hw = hwprobe.HardwareCollector()
        MockExists.side_effect = [False, True]
        with patch('rhsmlib.facts.hwprobe.platform'):
            MockOpen.return_value.readline.return_value = "this is not really a release file of any sort"
            expected = {
                'distribution.version': 'Unknown',
                'distribution.name': 'Unknown',
                'distribution.id': 'Unknown',
                'distribution.version.modifier': ''
            }
            self.assertEqual(hw.get_release_info(), expected)

    @patch("os.path.exists")
    @patch(OPEN_FUNCTION)
    def test_distro(self, MockOpen, MockExists):
        MockExists.side_effect = [False, True]
        hw = hwprobe.HardwareCollector()
        MockOpen.return_value.readline.return_value = "Awesome OS release 42 (Go4It)"
        expected = {
            'distribution.version': '42',
            'distribution.name': 'Awesome OS',
            'distribution.id': 'Go4It',
            'distribution.version.modifier': ''
        }
        self.assertEqual(hw.get_release_info(), expected)

    @patch("os.path.exists")
    @patch(OPEN_FUNCTION)
    def test_distro_newline_in_release(self, MockOpen, MockExists):
        hw = hwprobe.HardwareCollector()
        MockExists.side_effect = [False, True]
        MockOpen.return_value.readline.return_value = "Awesome OS release 42 (Go4It)\n\n"
        expected = {
            'distribution.version': '42',
            'distribution.name': 'Awesome OS',
            'distribution.id': 'Go4It',
            'distribution.version.modifier': ''
        }
        self.assertEqual(hw.get_release_info(), expected)

    @patch("os.path.exists")
    @patch(OPEN_FUNCTION)
    def test_manual_distro_bogus_content_os_release(self, MockOpen, MockExists):
        hw = hwprobe.HardwareCollector()
        with patch('rhsmlib.facts.hwprobe.platform'):
            MockExists.return_value = True
            MockOpen.return_value.readlines.return_value = ["This is not really a release file of any sort"]
            expected = {
                'distribution.version': 'Unknown',
                'distribution.name': 'Unknown',
                'distribution.id': 'Unknown',
                'distribution.version.modifier': ''
            }
            self.assertEqual(hw.get_release_info(), expected)

    @patch("os.path.exists")
    @patch(OPEN_FUNCTION, mock_open(read_data="Awesome OS release 42 Mega (Go4It)"))
    def test_distro_with_redhat_release(self, mock_exists):
        """
        Test reading release information from /etc/redhat-release
        """

        def mock_os_path_exists(path):
            """
            This method mock missing /etc/os-release
            :param path: of this file
            :return: False for /etc/os-release and True for any other file
            """
            if path == '/etc/os-release':
                return False
            else:
                return True

        mock_exists.side_effect = mock_os_path_exists
        hw = hwprobe.HardwareCollector()
        expected = {
            'distribution.version': '42',
            'distribution.name': 'Awesome OS',
            'distribution.id': 'Go4It',
            'distribution.version.modifier': 'mega'
        }
        self.assertEqual(hw.get_release_info(), expected)

    @patch("os.path.exists")
    @patch(OPEN_FUNCTION, mock_open(read_data="""
NAME=Awesome OS
VERSION="42 (Go4It)"
ID=awesome
VERSION_ID=42
VERSION_CODENAME=""
PLATFORM_ID="platform:a42"
PRETTY_NAME="Awesome OS 42 (Go4It)"
ANSI_COLOR="0;34"
LOGO=awesome-logo-icon
CPE_NAME="cpe:/o:awesomeproject:awesome:32"
HOME_URL="https://awesomeproject.org/"
DOCUMENTATION_URL="https://docs.awesomeproject.org/en-US/awesome/a42/system-administrators-guide/"
SUPPORT_URL="https://awesomeproject.org/wiki/Communicating_and_getting_help"
BUG_REPORT_URL="https://bugzilla.awesomeproject.org/"
PRIVACY_POLICY_URL="https://awesomeproject.org/wiki/Legal:PrivacyPolicy"
VARIANT="Go4It"
VARIANT_ID=server
    """))
    def test_distro_with_os_release(self, mock_exists):
        """
        Test reading release information from /etc/os-release
        """

        def mock_os_path_exists(path):
            """
            This method mock missing /etc/redhat-release
            :param path: of this file
            :return: False for /etc/redhat-release and True for any other file
            """
            if path == '/etc/redhat-release':
                return False
            else:
                return True

        mock_exists.side_effect = mock_os_path_exists
        hw = hwprobe.HardwareCollector()
        expected = {
            'distribution.version': '42',
            'distribution.name': 'Awesome OS',
            'distribution.id': 'Go4It',
            'distribution.version.modifier': ''
        }
        self.assertEqual(hw.get_release_info(), expected)

    @patch("os.path.exists")
    @patch(OPEN_FUNCTION)
    def test_manual_distro_with_modifier(self, MockOpen, MockExists):
        MockExists.side_effect = [False, True]
        hw = hwprobe.HardwareCollector()
        with patch('rhsmlib.facts.hwprobe.platform'):
            MockOpen.return_value.readline.return_value = "Awesome OS release 42 Mega (Go4It)"
            expected = {
                'distribution.version': '42',
                'distribution.name': 'Awesome OS',
                'distribution.id': 'Go4It',
                'distribution.version.modifier': 'mega'
            }
            self.assertEqual(hw.get_release_info(), expected)

    @patch("os.path.exists")
    @patch(OPEN_FUNCTION)
    def test_distro_os_release(self, MockOpen, MockExists):
        MockExists.return_value = True
        hw = hwprobe.HardwareCollector()
        with patch('rhsmlib.facts.hwprobe.platform'):
            MockOpen.return_value.readlines.return_value = OS_RELEASE.split('\n')
            expected = {
                'distribution.version': '42',
                'distribution.name': 'Awesome OS',
                'distribution.id': 'Go4It',
                'distribution.version.modifier': 'beta'
            }
            self.assertEqual(hw.get_release_info(), expected)

    @patch("os.path.exists")
    @patch(OPEN_FUNCTION)
    def test_distro_os_release_colon(self, MockOpen, MockExists):
        MockExists.return_value = True
        hw = hwprobe.HardwareCollector()
        with patch('rhsmlib.facts.hwprobe.platform'):
            MockOpen.return_value.readlines.return_value = OS_RELEASE_COLON.split('\n')
            expected = {
                'distribution.version': '42',
                'distribution.name': 'Awesome OS',
                'distribution.id': 'Go4It',
                'distribution.version.modifier': 'be:ta'
            }
            self.assertEqual(hw.get_release_info(), expected)

    def test_meminfo(self):
        hw = hwprobe.HardwareCollector()
        mem = hw.get_mem_info()
        # not great tests, but alas
        self.assertEqual(len(mem), 2)
        for key in mem:
            assert key in ['memory.memtotal', 'memory.swaptotal']

    # this test will probably fail on a machine with
    # no network.
    def test_networkinfo(self):
        hw = hwprobe.HardwareCollector()
        net = hw.get_network_info()
        expected = set(['network.fqdn', 'network.hostname', 'network.ipv4_address', 'network.ipv6_address'])
        self.assertEqual(expected, set(net.keys()))

    def test_network_interfaces(self):
        hw = hwprobe.HardwareCollector()
        net_int = hw.get_network_interfaces()
        self.assertEqual(net_int['net.interface.lo.ipv4_address'], '127.0.0.1')
        self.assertFalse('net.interface.lo.mac_address' in net_int)
        self.assertFalse('net.interface.sit0.mac_address' in net_int)

    # simulate some wacky interfaces
    @patch("ethtool.get_devices")
    @patch("ethtool.get_interfaces_info")
    def test_network_interfaces_none(self, MockGetInterfacesInfo, MockGetDevices):
        hw = hwprobe.HardwareCollector()
        net_int = hw.get_network_interfaces()
        self.assertEqual(net_int, {})

    @patch("ethtool.get_devices")
    @patch("ethtool.get_interfaces_info")
    def test_network_interfaces_multiple_ipv4(self, MockGetInterfacesInfo, MockGetDevices):
        hw = hwprobe.HardwareCollector()

        MockGetDevices.return_value = ['eth0']
        mock_info = Mock(mac_address="00:00:00:00:00:00", device="eth0")
        mock_info.get_ipv6_addresses.return_value = []
        mock_ipv4s = [Mock(address="10.0.0.1", netmask="24", broadcast="Unknown"),
                      Mock(address="10.0.0.2", netmask="24", broadcast="Unknown")]
        mock_info.get_ipv4_addresses = Mock(return_value=mock_ipv4s)
        MockGetInterfacesInfo.return_value = [mock_info]

        net_int = hw.get_network_interfaces()

        self.assertEqual(net_int['net.interface.eth0.ipv4_address'], '10.0.0.2')
        self.assertEqual(net_int['net.interface.eth0.ipv4_address_list'], '10.0.0.1, 10.0.0.2')

    @patch("ethtool.get_devices")
    @patch("ethtool.get_interfaces_info")
    def test_network_interfaces_just_lo(self, MockGetInterfacesInfo, MockGetDevices):
        hw = hwprobe.HardwareCollector()
        MockGetDevices.return_value = ['lo']
        mock_info = Mock(mac_address="00:00:00:00:00:00",
                         device="lo")

        mock_info.get_ipv6_addresses.return_value = []
        mock_ipv4 = Mock(address="127.0.0.1",
                         netmask="24",
                         broadcase="Unknown")
        mock_info.get_ipv4_addresses = Mock(return_value=[mock_ipv4])
        MockGetInterfacesInfo.return_value = [mock_info]
        net_int = hw.get_network_interfaces()
        self.assertEqual(net_int['net.interface.lo.ipv4_address'], '127.0.0.1')
        self.assertFalse('net.interface.lo.mac_address' in net_int)

    @patch("ethtool.get_devices")
    @patch("ethtool.get_interfaces_info")
    def test_network_interfaces_sit(self, MockGetInterfacesInfo, MockGetDevices):
        hw = hwprobe.HardwareCollector()
        MockGetDevices.return_value = ['sit0']
        mock_ipv6 = Mock(address="::1",
                         netmask="/128",
                         scope="global")

        mock_info = Mock(mac_address="00:00:00:00:00:00",
                         device="sit0")
        mock_info.get_ipv6_addresses.return_value = [mock_ipv6]
        mock_info.get_ipv4_addresses.return_value = []
        MockGetInterfacesInfo.return_value = [mock_info]

        net_int = hw.get_network_interfaces()
        # ignore mac address for sit* interfaces (bz #838123)
        self.assertFalse('net.interface.sit0.mac_address' in net_int)

    @patch("ethtool.get_devices")
    @patch("ethtool.get_interfaces_info")
    def test_network_interfaces_just_lo_ethtool_no_get_ipv4_addresses(self,
        MockGetInterfacesInfo, MockGetDevices):

        hw = hwprobe.HardwareCollector()
        MockGetDevices.return_value = ['lo']
        mock_info = Mock(mac_address="00:00:00:00:00:00",
                         device="lo",
                         ipv4_address="127.0.0.1",
                         ipv4_netmask="24",
                         ipv4_broadcast="Unknown")
        mock_info.get_ipv6_addresses.return_value = []

        # mock etherinfo not having a get_ipv4_addresses method
        # if this fails, you need a mock that supports deleting
        # attributes from mocks, ala mock 1.0+
        try:
            del mock_info.get_ipv4_addresses
        except AttributeError:
            self.fail("You probably need a newer version of 'mock' installed, 1.0 or newer")

        MockGetInterfacesInfo.return_value = [mock_info]

        net_int = hw.get_network_interfaces()
        self.assertEqual(net_int['net.interface.lo.ipv4_address'], '127.0.0.1')
        self.assertFalse('net.interface.lo.mac_address' in net_int)

    @patch("ethtool.get_devices")
    @patch("ethtool.get_interfaces_info")
    def test_network_interfaces_just_lo_ipv6(self, MockGetInterfacesInfo, MockGetDevices):
        hw = hwprobe.HardwareCollector()
        MockGetDevices.return_value = ['lo']

        mock_ipv6 = Mock(address="::1",
                         netmask="/128",
                         scope="global")

        mock_info = Mock(mac_address="00:00:00:00:00:00",
                         device="lo")
        mock_info.get_ipv6_addresses.return_value = [mock_ipv6]
        mock_info.get_ipv4_addresses.return_value = []
        MockGetInterfacesInfo.return_value = [mock_info]

        net_int = hw.get_network_interfaces()
        self.assertEqual(net_int['net.interface.lo.ipv6_address.global'], '::1')
        self.assertFalse('net.interface.lo.mac_address' in net_int)

    @patch(OPEN_FUNCTION)
    def test_get_slave_hwaddr_rr(self, MockOpen):
        MockOpen.return_value = six.StringIO(PROC_BONDING_RR)
        hw = hwprobe.HardwareCollector()
        slave_hw = hw._get_slave_hwaddr("bond0", "eth0")
        # note we .upper the result
        self.assertEqual("52:54:00:07:03:BA", slave_hw)

    @patch(OPEN_FUNCTION)
    def test_get_slave_hwaddr_alb(self, MockOpen):
        MockOpen.return_value = six.StringIO(PROC_BONDING_ALB)
        hw = hwprobe.HardwareCollector()
        slave_hw = hw._get_slave_hwaddr("bond0", "eth0")
        # note we .upper the result
        self.assertEqual("52:54:00:07:03:BA", slave_hw)

    def test_parse_s390_sysinfo_empty(self):
        cpu_count = 0
        sysinfo_lines = []

        ret = self.hw_check_topo._parse_s390x_sysinfo_topology(cpu_count, sysinfo_lines)
        self.assertTrue(ret is None)

    def test_parse_s390_sysinfo(self):
        cpu_count = 24
        sysinfo_lines = ["CPU Topology SW:      0 0 0 4 6 4"]

        ret = self.hw_check_topo._parse_s390x_sysinfo_topology(cpu_count, sysinfo_lines)

        self.assertEqual(24, ret['socket_count'])
        self.assertEqual(4, ret['book_count'])
        self.assertEqual(6, ret['sockets_per_book'])
        self.assertEqual(4, ret['cores_per_socket'])

    @patch(OPEN_FUNCTION, mock_open(read_data=PROC_STAT))
    def test_parse_proc_stat_btime(self):
        expected_btime = "1524062304"

        hw = hwprobe.HardwareCollector()
        ret = hw.get_proc_stat()
        self.assertEqual(expected_btime, ret['proc_stat.btime'])

    @patch.object(hwprobe.HardwareCollector, 'count_cpumask_entries')
    @patch("os.listdir")
    def test_cpu_info_s390(self, mock_list_dir, mock_mask):
        mock_list_dir.return_value = ["cpu%s" % i for i in range(0, 3)]

        # 32 cpus
        # 16 cores, 2 threads per core = each cpu has two thread siblings
        # 1 core per socket
        # 8 sockets per book, = each cpu has 8 core siblings
        # 2 books, each check has 16 book siblings
        def count_cpumask(cpu, field):
            cpumask_vals = {
                'thread_siblings_list': 1,
                'core_siblings_list': 1,
                'book_siblings_list': 1
            }
            return cpumask_vals[field]

        with patch.object(self.hw_check_topo, 'count_cpumask_entries', Mock(side_effect=count_cpumask)):
            expected = {
                'cpu.cpu(s)': 3,
                'cpu.socket(s)_per_book': 1,
                'cpu.core(s)_per_socket': 1,
                'cpu.thread(s)_per_core': 1,
                'cpu.cpu_socket(s)': 3,
                'cpu.book(s)': 3,
                'cpu.book(s)_per_cpu': 1,
                'cpu.topology_source': 's390 book_siblings_list'
            }

            self.assert_equal_dict(expected, self.hw_check_topo.get_cpu_info())

    @patch.object(hwprobe.HardwareCollector, 'has_s390x_sysinfo')
    @patch.object(hwprobe.HardwareCollector, 'read_s390x_sysinfo')
    @patch.object(hwprobe.HardwareCollector, 'check_for_cpu_topo')
    @patch("os.listdir")
    def test_cpu_info_s390_sysinfo(self, mock_list_dir, mock_topo, mock_read_sysinfo, mock_has_sysinfo):
        mock_list_dir.return_value = ["cpu%s" % i for i in range(0, 20)]
        mock_has_sysinfo.return_value = True
        mock_topo.return_value = True
        mock_read_sysinfo.return_value = ["CPU Topology SW:      0 0 0 4 6 4"]

        self.hw_check_topo.arch = 's390x'

        # 20 cpus
        # 24 cores, 1 threads per core
        # 1 thread per core, 1 core per socket, 1 socket per book via /sys, but
        # /proc/sysinfo says 4 books of 6 sockets of 4 cores
        #
        # even though we have cpu topo from sysinfo, we also have
        # info from the kernel, which we use in that case
        #
        # and we prefer /proc/sysinfo
        # how do 24 sockets have 20 cpu? The hardware itself
        # has 96 cpus. 54 of those are enabled at the
        # hardware level. Of those 51, 21 are "configured".
        # The LPAR is setup to use 20 of those 21 cpus
        # (and in this setup, actually only 18 of those
        # are "configured").
        def count_cpumask(cpu, field):
            cpumask_vals = {
                'thread_siblings_list': 1,
                'core_siblings_list': 1,
                'book_siblings_list': 1
            }
            return cpumask_vals[field]

        # for this case, we prefer the sysinfo numbers
        with patch.object(self.hw_check_topo, 'count_cpumask_entries', Mock(side_effect=count_cpumask)):
            expected = {
                'cpu.cpu(s)': 20,
                'cpu.socket(s)_per_book': 6,
                'cpu.core(s)_per_socket': 4,
                'cpu.thread(s)_per_core': 1,
                'cpu.book(s)': 4,
                'cpu.cpu_socket(s)': 24,
                'cpu.topology_source': 's390x sysinfo'
            }
            self.assert_equal_dict(expected, self.hw_check_topo.get_cpu_info())

    @patch.object(hwprobe.HardwareCollector, 'count_cpumask_entries')
    @patch("os.listdir")
    def test_cpu_info(self, mock_list_dir, mock_count):
        def count_cpumask(cpu, field):
            cpumask_vals = {
                'thread_siblings_list': 1,
                'core_siblings_list': 2,
                'book_siblings_list': None
            }
            return cpumask_vals[field]

        mock_list_dir.return_value = ["cpu0", "cpu1"]
        with patch.object(self.hw_check_topo, 'count_cpumask_entries', Mock(side_effect=count_cpumask)):
            expected = {
                'cpu.cpu(s)': 2,
                'cpu.core(s)_per_socket': 2,
                'cpu.cpu_socket(s)': 1,
                'cpu.thread(s)_per_core': 1,
                'cpu.topology_source': 'kernel /sys cpu sibling lists'
            }
            self.assert_equal_dict(expected, self.hw_check_topo.get_cpu_info())

    @patch("os.listdir")
    def test_cpu_info_no_topo(self, mock_list_dir):
        def count_cpumask(cpu, field):
            cpumask_vals = {
                'thread_siblings_list': None,
                'core_siblings_list': None,
                'book_siblings_list': None
            }
            return cpumask_vals[field]

        mock_list_dir.return_value = ["cpu%s" % i for i in range(0, 16)]

        with patch.object(self.hw_check_topo, 'count_cpumask_entries', Mock(side_effect=count_cpumask)):
            expected = {
                'cpu.cpu(s)': 16,
                'cpu.core(s)_per_socket': 1,
                'cpu.cpu_socket(s)': 16,
                'cpu.thread(s)_per_core': 1,
                'cpu.topology_source': "fallback one socket"
            }
            self.assert_equal_dict(expected, self.hw_check_topo.get_cpu_info())

    @patch.object(hwprobe.HardwareCollector, "read_physical_id")
    @patch("os.listdir")
    def test_cpu_info_no_topo_ppc64_physical_id(self, mock_list_dir,
                                                mock_read_physical):
        self.hw_check_topo.arch = "ppc64"

        def get_physical(cpu_file):
            # pretend we have two physical package ids
            return int(cpu_file[-1]) % 2

        def count_cpumask(cpu, field):
            cpumask_vals = {
                'thread_siblings_list': None,
                'core_siblings_list': None,
                'book_siblings_list': None
            }
            return cpumask_vals[field]

        mock_list_dir.return_value = ["cpu%s" % i for i in range(0, 8)]
        with patch.object(self.hw_check_topo, 'count_cpumask_entries', Mock(side_effect=count_cpumask)):
            with patch.object(self.hw_check_topo, 'read_physical_id', Mock(side_effect=get_physical)):
                expected = {
                    'cpu.cpu(s)': 8,
                    'cpu.core(s)_per_socket': 4,
                    'cpu.cpu_socket(s)': 2,
                    'cpu.thread(s)_per_core': 1,
                    'cpu.topology_source': 'ppc64 physical_package_id'
                }
                self.assert_equal_dict(expected, self.hw_check_topo.get_cpu_info())

    @patch("os.listdir")
    def test_cpu_info_lots_cpu(self, mock_list_dir):
        mock_list_dir.return_value = ["cpu%s" % i for i in range(0, 2000)]

        def count_cpumask(cpu, field):
            vals = {
                'thread_siblings_list': 1,
                #'core_siblings_list': 2,
                'core_siblings_list': 2000,
                'book_siblings_list': None
            }
            return vals[field]

        with patch.object(self.hw_check_topo, 'count_cpumask_entries', Mock(side_effect=count_cpumask)):
            expected = {
                'cpu.cpu(s)': 2000,
                'cpu.core(s)_per_socket': 2000,
                'cpu.thread(s)_per_core': 1,
                'cpu.cpu_socket(s)': 1,
                'cpu.topology_source': 'kernel /sys cpu sibling lists'
            }
            self.assert_equal_dict(expected, self.hw_check_topo.get_cpu_info())

    @patch("os.listdir")
    def test_cpu_info_other_files(self, mock_list_dir):
        mock_list_dir.return_value = [
            "cpu0", "cpu1",  # normal cpu ids (valid)
            "cpu123123",     # big cpu   (valid)
            "cpu_",          # not valid
            "cpufreq",       # this exists but is not a cpu
            "cpuidle",       # also exists
            "cpu0foo",       # only cpuN are valid
            "cpu11111111 ",  # trailing space, not valie
            "cpu00"          # odd name, but valid I guess
        ]

        def count_cpumask(cpu, field):
            vals = {
                'thread_siblings_list': 1,
                #'core_siblings_list': 2,
                'core_siblings_list': 4,
                'book_siblings_list': None
            }
            return vals[field]

        with patch.object(self.hw_check_topo, 'count_cpumask_entries', Mock(side_effect=count_cpumask)):
            expected = {
                'cpu.cpu(s)': 4,
                'cpu.core(s)_per_socket': 4,
                'cpu.thread(s)_per_core': 1,
                'cpu.cpu_socket(s)': 1,
                'cpu.topology_source': 'kernel /sys cpu sibling lists'
            }
            self.assert_equal_dict(expected, self.hw_check_topo.get_cpu_info())


class TestLscpu(unittest.TestCase):
    @patch('os.environ', {
        'LANGUAGE': 'ja_JP.eucJP',
        'LC_ALL': 'ja_JP.eucJP',
        'LC_CTYPE': 'ja_JP.eucJP',
        'LANG': 'ja_JP.eucJP',
    })
    def test_lscpu_ignores_locale(self):
        hw_check_topo = hwprobe.HardwareCollector()
        facts = hw_check_topo.get_ls_cpu_info()
        # if all values can be encoded as ascii, then lscpu is not using JP locale
        for key, value in facts.items():
            key.encode('ascii')
            value.encode('ascii')



from __future__ import print_function, division, absolute_import

#
# Copyright (c) 2015 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public
# License as published by the Free Software Foundation; either version
# 2 of the License (GPLv2) or (at your option) any later version.
# There is NO WARRANTY for this software, express or implied,
# including the implied warranties of MERCHANTABILITY,
# NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
# have received a copy of GPLv2 along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
"""
Class and methods made available in __all__ are provided for use by
external applications.

All reasonable efforts will be made to maintain compatibility.
"""
from functools import wraps
from rhsm import logutil

injected = False


def request_injection(func):
    """This idempotent decorator can be applied to initialize the dependency
    injection used by subscription manager.  Users of the API methods will not
    normally need to use this decorator as it will already have been called."""
    @wraps(func)
    def func_wrapper(*args, **kwargs):
        global injected
        if not injected:
            logutil.init_logger()

            from subscription_manager.injectioninit import init_dep_injection
            init_dep_injection()
            injected = True
        return func(*args, **kwargs)
    return func_wrapper


from .repos import disable_yum_repositories, enable_yum_repositories

from subscription_manager.version import rpm_version as version

__all__ = [
    'request_injection',
    'disable_yum_repositories',
    'enable_yum_repositories',
    'version',
]



from __future__ import print_function, division, absolute_import

# Copyright (c) 2016 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#

# A compatibility wrapper that adapts m2crypto to the subset of standard
# lib ssl used in python-rhsm.

from M2Crypto import m2 as _m2, SSL as _ssl

CERT_NONE = _ssl.verify_none
CERT_REQUIRED = _ssl.verify_peer | _ssl.verify_fail_if_no_peer_cert
PROTOCOL_SSLv23 = 'sslv23'
OP_ALL = _m2.SSL_OP_ALL
OP_NO_SSLv2 = _m2.SSL_OP_NO_SSLv2
OP_NO_SSLv3 = _m2.SSL_OP_NO_SSLv3


class NoOpChecker(object):
    def __init__(self, host=None, peerCertHash=None, peerCertDigest='sha1'):
        self.host = host
        self.fingerprint = peerCertHash
        self.digest = peerCertDigest

    def __call__(self, peerCert, host=None):
        return True


class SSLContext(object):

    def __init__(self, version):
        self.m2context = _ssl.Context(version)
        self._options = None
        self._default_verify_depth = self.m2context.get_verify_depth()
        self.options = OP_ALL
        self.verify_mode = CERT_NONE
        self.check_hostname = False

    @property
    def check_hostname(self):
        return self._check_hostname

    @check_hostname.setter
    def check_hostname(self, check_hostname):
        self._check_hostname = check_hostname
        if not check_hostname:
            self.m2context.post_connection_check = NoOpChecker()
        else:
            self.m2context.post_connection_check = None

    @property
    def options(self):
        return self._options

    @options.setter
    def options(self, options):
        self._options = options
        self.m2context.set_options(options)

    @property
    def verify_mode(self):
        return self.m2context.get_verify_mode()

    @verify_mode.setter
    def verify_mode(self, verify_mode):
        self.m2context.set_verify(verify_mode, self._default_verify_depth)

    def load_verify_locations(self, cafile=None, capath=None, cadata=None):
        if cadata:
            raise NotImplementedError
        return self.m2context.load_verify_locations(cafile, capath)

    def load_cert_chain(self, certfile, keyfile=None, password=None):
        if password:
            return self.m2context.load_cert(certfile, keyfile, lambda: password)
        else:
            return self.m2context.load_cert(certfile, keyfile)

SSLError = _ssl.SSLError



from __future__ import print_function, division, absolute_import

#
# Copyright (c) 2015 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#

import types
import os
import sys

import logging
log = logging.getLogger(__name__)

from subscription_manager import version

""" Try importing from a 'ga' that is dynamic and imports
    the gi.repository style names into the module. This
    uses python's sys.metapath support to add a custom
    importer to the "front" of sys.path. The custom
    importer knows how to handle imports for
    'subscription_manager.ga'. The importer is in the
    ga_loader module, and is used to decide which impl
    of 'ga' is used (gtk2 or gtk3).

    Currently doing 'from subscription_manager.ga import GObject as ga_Gobject'
    since for gtk2 cases, it's not really GObject, and we avoid
    shadowing well known names, but maybe it doesn't matter.

    The 'virtual' module 'ga' will import the correct
    implementation from the ga_impls/ dir. ga_loader
    decides which version to use. It currently defaults
    to gtk3, with an env variable to override for testing.

"""


class GaImporter(object):
    """Custom module importer protocol that finds and loads the 'ga' Gtk2/Gtk3 compat virtual module.

    This implements the module "Importer Protocol" as defined
    in PEP302 (https://www.python.org/dev/peps/pep-0302/). It provides
    both a module finder (find_modules) and a module loader (load_modules).

    This lets different sub classes of this module to all provide a set of
    module names in the 'ga' namespace, but to provide different implementations.

    When an instance of this class is added to sys.meta_path, all imports that
    reference modules by name (ie, normal 'import bar' statements) the names are first passed
    to this classes 'find_module' method. When this class is asked for modules in
    the 'ga' package, it returns itself (which is also a module loader).

    This classes load_module() is used to decide which implemention of the 'ga'
    package to load. GaImporter.virtual_modules is a dict mapping full module name
    to the full name of the module that is to be loaded for that name.

    The 'ga' module implementations are in the ga_impls/ module.
    The available implementations are 'ga_gtk2' and 'ga_gtk3'.

    The 'ga' module itself provides a Gtk3-like API.

    The 'ga_impls/ga_gtk3' implementation is an export of the full 'gi.repository.Gtk',
    and a few helper methods and names.

    The 'ga_impls/ga_gtk2' implementation is more complicated. It maps a subset of
    Gtk2 names and widgets to their Gtk3 equilivent. This includes an assortment
    of class enums, and helper methods. The bulk of the API compat is just mapping
    names like 'gtk.Window' to Gtk style names like 'gi.repository.Gtk.Window'.

    NOTE: Only symbols actually used in subscription-manager are provided. This
          is not a general purpose Gtk3 interface for Gtk2. Names are imported
          directly and export directly in module __all__ attributes. This is to
          make sure any Gtk3 widgets used in subman have working gtk2 equilivents
          and ga_gtk2 provides it.
    """

    namespace = "subscription_manager.ga"
    virtual_modules = {}

    def __init__(self):
        log.debug("ga_loader %s", self.__class__.__name__)

    def find_module(self, fullname, path):
        if fullname in self.virtual_modules:
            return self
        return None

    def load_module(self, fullname):
        if fullname in sys.modules:
            return sys.modules[fullname]

        if fullname not in self.virtual_modules:
            raise ImportError(fullname)

        # The base namespace
        if fullname == self.namespace:
            return self._namespace_module()

        real_module_name = real_module_from = None
        mod_info = self.virtual_modules[fullname]
        if mod_info:
            real_module_name, real_module_from = mod_info

        if not real_module_from:
            raise ImportError(fullname)

        # looks like a real_module alias
        return self._import_real_module(fullname, real_module_name, real_module_from)

    def _import_real_module(self, fullname, module_name, module_from):
        ret = __import__(module_name, globals(), locals(), [module_from])
        inner_ret = getattr(ret, module_from)
        ret = inner_ret
        ret.__name__ = fullname
        ret.__loader__ = self
        ret.__package__ = True
        sys.modules[fullname] = ret
        return ret

    def _new_module(self, fullname):
        """Create a an empty module, we can populate with impl specific."""
        ret = sys.modules.setdefault(fullname, types.ModuleType(fullname))
        ret.__name__ = fullname
        ret.__loader__ = self
        ret.__filename__ = fullname
        ret.__path__ = [fullname]
        ret.__package__ = '.'.join(fullname.split('.')[:-1])
        return ret

    def _namespace_module(self):
        """Create and return a 'ga' package module.

        Since the 'ga' module has to work for Gtk2/Gtk3, but can't import
        either, we create a new module instance and add it to the system
        path.

        Imports like 'from ga import Gtk3' first have to import 'ga'. When
        they do, the module instance is the one we create here.
        """
        return self._new_module(self.namespace)


class GaImporterGtk3(GaImporter):
    virtual_modules = {'subscription_manager.ga': None,
                       'subscription_manager.ga.gtk_compat': ['subscription_manager.ga_impls',
                                                              'ga_gtk3'],
                       'subscription_manager.ga.GObject': ['gi.repository',
                                                           'GObject'],
                       'subscription_manager.ga.Gdk': ['gi.repository',
                                                       'Gdk'],
                       'subscription_manager.ga.Gtk': ['gi.repository',
                                                       'Gtk'],
                       'subscription_manager.ga.GLib': ['gi.repository',
                                                        'GLib'],
                       'subscription_manager.ga.GdkPixbuf': ['gi.repository',
                                                             'GdkPixbuf'],
                       'subscription_manager.ga.Pango': ['gi.repository',
                                                         'Pango']}


class GaImporterGtk2(GaImporter):
    virtual_modules = {'subscription_manager.ga': None,
                       'subscription_manager.ga.gtk_compat': ['subscription_manager.ga_impls.ga_gtk2',
                                                              'gtk_compat'],
                       'subscription_manager.ga.GObject': ['subscription_manager.ga_impls.ga_gtk2',
                                                           'GObject'],
                       'subscription_manager.ga.Gdk': ['subscription_manager.ga_impls.ga_gtk2',
                                                       'Gdk'],
                       'subscription_manager.ga.Gtk': ['subscription_manager.ga_impls.ga_gtk2',
                                                       'Gtk'],
                       'subscription_manager.ga.GLib': ['subscription_manager.ga_impls.ga_gtk2',
                                                        'GLib'],
                       'subscription_manager.ga.GdkPixbuf': ['subscription_manager.ga_impls.ga_gtk2',
                                                             'GdkPixbuf'],
                       'subscription_manager.ga.Pango': ['subscription_manager.ga_impls.ga_gtk2',
                                                         'Pango']}


def init_ga(gtk_version=None):
    """Decide which GaImporter implementation to load.

    Applications should import this module and call this method before
    importing anything from the 'ga' namespace.

    After calling this method, a GaImporter implementation is added to sys.meta_path.
    This sets up a module finder and loader that will return 'virtual' modules
    when asked for 'ga.Gtk' for example. Depending on the GaImporter, 'ga.Gtk'
    may be implemented with Gtk3 or gtk2.

    The default implementation is the gtk2 based one (DEFAULT_GTK_VERSION).

    The acceptable values of 'gtk_version' are '2' and '3', for gtk2 and
    gtk3.

    It can be overridden by, in order:

        Hardcoded DEFAULT_GTK_VERSION.
        (default is '2')

        The value of subscription_manager.version.gtk_version if it exists
        and is not None.
        (As set at build time)

        The 'gtk_version' argument to this method if not None.
        (The default is None)

        The value of the environment variable 'SUBMAN_GTK_VERSION' if set
        to '2' or '3'.
        (default is unset)
    """

    DEFAULT_GTK_VERSION = "2"
    gtk_version_from_build = None
    gtk_version_from_environ = os.environ.get('SUBMAN_GTK_VERSION')

    # ignore version.py info if it hasn't been set.
    if version.gtk_version != "GTK_VERSION":
        gtk_version_from_build = version.gtk_version
    else:
        # We should only be hitting this code in development when the version.py
        # hasn't been through the setup.py rendering process.
        import rpm
        import warnings
        # 0 if %rhel is undefined, rhel version if not
        rhel = rpm.expandMacro('%rhel', True)

        if rhel and rhel == 6:
            gtk_version_from_build = "2"
        else:
            gtk_version_from_build = "3"
        if gtk_version_from_environ is None:
            warnings.warn("GTK_VERSION is unset in version.py.  Using GTK %s" % gtk_version_from_build)

    GTK_VERSION = gtk_version_from_environ or gtk_version or gtk_version_from_build \
        or DEFAULT_GTK_VERSION

    if GTK_VERSION == "3":
        sys.meta_path.append(GaImporterGtk3())
    if GTK_VERSION == "2":
        sys.meta_path.append(GaImporterGtk2())



from __future__ import print_function, division, absolute_import

# Copyright (c) 2010-2013 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#
"""Load and collect DMI data.

Note: This module will fail to import if dmidecode fails to import.
      firmware_info.py expects that and handles it, and any other
      module that imports it should handle an import error as well."""
import logging
import os
import six

from subscription_manager.i18n import ugettext as _

log = logging.getLogger(__name__)

from rhsmlib.facts import collector

FIRMWARE_DUMP_FILENAME = "dmi.dump"


class DmiFirmwareInfoCollector(collector.FactsCollector):
    def __init__(self, prefix=None, testing=None, collected_hw_info=None):
        super(DmiFirmwareInfoCollector, self).__init__(
            prefix=prefix,
            testing=testing,
            collected_hw_info=collected_hw_info
        )

        self._socket_designation = []
        self._socket_counter = 0

        self.dump_file = None
        if testing and prefix:
            self.dump_file = os.path.join(prefix, FIRMWARE_DUMP_FILENAME)

    def use_dump_file(self, dmidecode):
        """Set this instances to use a dmidecode dump file.

        WARNING: This involves settings a module global
        attribute in 'dmidecode', not just for this class
        or object, but for the lifetime of the dmidecode module.

        To 'unset' it, it can be set back to '/dev/mem', or
        re set it to another dump file."""
        if os.access(self.dump_file, os.R_OK):
            dmidecode.set_dev(self.dump_file)

    # This needs all of the previously collected hwinfo, so it can decide
    # what is bogus enough that the DMI info is better.
    def get_all(self):
        try:
            import dmidecode
        except ImportError:
            log.warn("Unable to load dmidecode module. No DMI info will be collected")
            raise

        dmiinfo = {}
        try:
            # When alternative memory device file was specified for this class, then
            # try to use it. Otherwise current device file will be used.
            if self.dump_file is not None:
                self.use_dump_file(dmidecode)
            log.debug("Using dmidecode dump file: %s" % dmidecode.get_dev())
            dmi_data = {
                "dmi.bios.": self._read_dmi(dmidecode.bios),
                "dmi.processor.": self._read_dmi(dmidecode.processor),
                "dmi.baseboard.": self._read_dmi(dmidecode.baseboard),
                "dmi.chassis.": self._read_dmi(dmidecode.chassis),
                "dmi.slot.": self._read_dmi(dmidecode.slot),
                "dmi.system.": self._read_dmi(dmidecode.system),
                "dmi.memory.": self._read_dmi(dmidecode.memory),
                "dmi.connector.": self._read_dmi(dmidecode.connector),
            }

            for tag, func in list(dmi_data.items()):
                dmiinfo = self._get_dmi_data(func, tag, dmiinfo)
        except Exception as e:
            log.warn(_("Error reading system DMI information: %s"), e, exc_info=True)
        finally:
            self.log_warnings(dmidecode)
        return dmiinfo

    def _read_dmi(self, func):
        try:
            return func()
        except Exception as e:
            log.warn(_("Error reading system DMI information with %s: %s"), func, e)
            return {}

    def _get_dmi_data(self, func, tag, ddict):
        for key, value in list(func.items()):
            for key1, value1 in list(value['data'].items()):
                # FIXME: this loses useful data...
                if not isinstance(value1, six.text_type) and not isinstance(value1, six.binary_type):
                    # we are skipping things like int and bool values, as
                    # well as lists and dicts
                    continue

                # keep track of any cpu socket info we find, we have to do
                # it here, since we flatten it and lose the info creating nkey
                if tag == 'dmi.processor.' and key1 == 'Socket Designation':
                    self._socket_designation.append(value1)

                nkey = ''.join([tag, key1.lower()]).replace(" ", "_")
                ddict[nkey] = six.text_type(value1, 'utf-8')

        # Populate how many socket descriptions we saw in a faux-fact, so we can
        # use it to munge lscpu info later if needed.
        if self._socket_designation:
            ddict['dmi.meta.cpu_socket_count'] = str(len(self._socket_designation))

        return ddict

    def log_warnings(self, dmidecode):
        dmiwarnings = dmidecode.get_warnings()
        if dmiwarnings:
            log.warn(_("Error reading system DMI information: %s"), dmiwarnings, exc_info=True)
            dmidecode.clear_warnings()



from __future__ import print_function, division, absolute_import

import os
import random
from . import fixture
from . import stubs

from tempfile import mkdtemp

from mock import Mock, patch
from rhsm.utils import ServerUrlParseErrorEmpty, \
    ServerUrlParseErrorNone, ServerUrlParseErrorPort, \
    ServerUrlParseErrorScheme, ServerUrlParseErrorJustScheme
from subscription_manager.utils import parse_server_info, \
    parse_baseurl_info, format_baseurl, \
    get_version, get_client_versions, unique_list_items, \
    get_server_versions, friendly_join, is_true_value, url_base_join,\
    ProductCertificateFilter, EntitlementCertificateFilter,\
    is_simple_content_access, is_process_running, get_process_names
from .stubs import StubProductCertificate, StubProduct, StubEntitlementCertificate
from .fixture import SubManFixture

from rhsm.config import DEFAULT_PORT, DEFAULT_PREFIX, DEFAULT_HOSTNAME, \
    DEFAULT_CDN_HOSTNAME, DEFAULT_CDN_PORT, DEFAULT_CDN_PREFIX

from rhsmlib.services import config


class TestParseServerInfo(SubManFixture):
    def setUp(self):
        SubManFixture.setUp(self)
        self.stubConfig = stubs.StubConfig()

    def test_fully_specified(self):
        local_url = "myhost.example.com:900/myapp"
        (hostname, port, prefix) = parse_server_info(local_url)
        self.assertEqual("myhost.example.com", hostname)
        self.assertEqual("900", port)
        self.assertEqual("/myapp", prefix)

    def test_hostname_only(self):
        local_url = "myhost.example.com"
        (hostname, port, prefix) = parse_server_info(local_url)
        self.assertEqual("myhost.example.com", hostname)
        self.assertEqual("443", port)
        self.assertEqual(DEFAULT_PREFIX, prefix)

    def test_hostname_port(self):
        local_url = "myhost.example.com:500"
        (hostname, port, prefix) = parse_server_info(local_url)
        self.assertEqual("myhost.example.com", hostname)
        self.assertEqual("500", port)
        self.assertEqual(DEFAULT_PREFIX, prefix)

    def test_hostname_prefix(self):
        local_url = "myhost.example.com/myapp"
        (hostname, port, prefix) = parse_server_info(local_url)
        self.assertEqual("myhost.example.com", hostname)
        self.assertEqual("443", port)
        self.assertEqual("/myapp", prefix)

    def test_hostname_only_config(self):
        self.stubConfig.set("server", "port", "344")
        self.stubConfig.set("server", "prefix", "/test-prefix")

        local_url = "myhost.example.com"
        (hostname, port, prefix) = parse_server_info(local_url, config.Config(self.stubConfig))
        self.assertEqual("myhost.example.com", hostname)
        self.assertEqual("344", port)
        self.assertEqual("/test-prefix", prefix)

    def test_hostname_port_config(self):
        self.stubConfig.set("server", "port", "600")

        local_url = "myhost.example.com/myapp"
        (hostname, port, prefix) = parse_server_info(local_url, config.Config(self.stubConfig))
        self.assertEqual("myhost.example.com", hostname)
        self.assertEqual("600", port)
        self.assertEqual("/myapp", prefix)

    def test_hostname_prefix_config(self):
        self.stubConfig.set("server", "prefix", "/test-prefix")

        local_url = "myhost.example.com:500"
        (hostname, port, prefix) = parse_server_info(local_url, config.Config(self.stubConfig))
        self.assertEqual("myhost.example.com", hostname)
        self.assertEqual("500", port)
        self.assertEqual("/test-prefix", prefix)

    def test_hostname_slash_no_prefix(self):
        local_url = "http://myhost.example.com/"
        (hostname, port, prefix) = parse_server_info(local_url)
        self.assertEqual("myhost.example.com", hostname)
        self.assertEqual("443", port)
        self.assertEqual("/", prefix)

    def test_hostname_just_slash(self):
        local_url = "/"
        (hostname, port, prefix) = parse_server_info(local_url)
        self.assertEqual(DEFAULT_HOSTNAME, hostname)
        self.assertEqual(DEFAULT_PORT, port)
        self.assertEqual("/", prefix)

    def test_hostname_nested_prefix(self):
        local_url = "myhost.example.com/myapp/subapp"
        (hostname, port, prefix) = parse_server_info(local_url)
        self.assertEqual("myhost.example.com", hostname)
        self.assertEqual("443", port)
        self.assertEqual("/myapp/subapp", prefix)

    def test_hostname_nothing(self):
        local_url = ""
        self.assertRaises(ServerUrlParseErrorEmpty,
                          parse_server_info,
                          local_url)

    def test_hostname_none(self):
        local_url = None
        self.assertRaises(ServerUrlParseErrorNone,
                          parse_server_info,
                          local_url)

    def test_hostname_with_scheme(self):
        # this is the default, so test it here
        local_url = "https://subscription.rhsm.redhat.com/subscription"
        (hostname, port, prefix) = parse_server_info(local_url)
        self.assertEqual("subscription.rhsm.redhat.com", hostname)
        self.assertEqual(DEFAULT_PORT, port)
        self.assertEqual("/subscription", prefix)

    def test_hostname_with_scheme_no_prefix(self):
        local_url = "https://myhost.example.com"
        (hostname, port, prefix) = parse_server_info(local_url)
        self.assertEqual("myhost.example.com", hostname)
        self.assertEqual(DEFAULT_PORT, port)
        self.assertEqual("/subscription", prefix)

    def test_hostname_no_scheme_port_no_prefix(self):
        local_url = "myhost.example.com:8443"
        (hostname, port, prefix) = parse_server_info(local_url)
        self.assertEqual("myhost.example.com", hostname)
        self.assertEqual("8443", port)
        self.assertEqual("/subscription", prefix)

    def test_just_prefix(self):
        local_url = "/myapp"
        (hostname, port, prefix) = parse_server_info(local_url)
        self.assertEqual(DEFAULT_HOSTNAME, hostname)
        self.assertEqual(DEFAULT_PORT, port)
        self.assertEqual("/myapp", prefix)

    def test_short_name(self):
        # could argue anything could be a local hostname, and we should
        # use default port and path. You could also argue it should
        # throw an error, especially if it's not a valid hostname
        local_url = "a"
        (hostname, port, prefix) = parse_server_info(local_url)
        self.assertEqual("a", hostname)
        self.assertEqual(DEFAULT_PORT, port)
        self.assertEqual(DEFAULT_PREFIX, prefix)

    def test_wrong_scheme(self):
        local_url = "git://git.fedorahosted.org/candlepin.git"
        self.assertRaises(ServerUrlParseErrorScheme,
                          parse_server_info,
                          local_url)

    def test_bad_http_scheme(self):
        # note missing /
        local_url = "https:/myhost.example.com:8443/myapp"
        self.assertRaises(ServerUrlParseErrorScheme,
                          parse_server_info,
                          local_url)

    def test_colon_but_no_port(self):
        local_url = "https://myhost.example.com:/myapp"
        self.assertRaises(ServerUrlParseErrorPort,
                          parse_server_info,
                          local_url)

    def test_colon_but_no_port_no_scheme(self):
        local_url = "myhost.example.com:/myapp"
        self.assertRaises(ServerUrlParseErrorPort,
                          parse_server_info,
                          local_url)

    def test_colon_slash_slash_but_nothing_else(self):
        local_url = "http://"
        self.assertRaises(ServerUrlParseErrorJustScheme,
                          parse_server_info,
                          local_url)

    def test_colon_slash_but_nothing_else(self):
        local_url = "http:/"
        self.assertRaises(ServerUrlParseErrorScheme,
                          parse_server_info,
                          local_url)

    def test_colon_no_slash(self):
        local_url = "http:example.com/foobar"
        self.assertRaises(ServerUrlParseErrorScheme,
                          parse_server_info,
                          local_url)

    # Note: this means if you have a local server named
    # "http", and you like redundant slashes, this actually
    # valid url of http//path/to/something will fail.
    # Don't do that. (or just use a single slash like http/path)
    # But seriously, really?
    def test_no_colon_double_slash(self):
        local_url = "http//example.com/api"
        self.assertRaises(ServerUrlParseErrorScheme,
                          parse_server_info,
                          local_url)

    def test_https_no_colon_double_slash(self):
        local_url = "https//example.com/api"
        self.assertRaises(ServerUrlParseErrorScheme,
                          parse_server_info,
                          local_url)

    # fail at internet
    def test_just_colon_slash(self):
        local_url = "://"
        self.assertRaises(ServerUrlParseErrorScheme,
                          parse_server_info,
                          local_url)

    def test_one_slash(self):
        local_url = "http/example.com"
        self.assertRaises(ServerUrlParseErrorScheme,
                          parse_server_info,
                          local_url)

    def test_host_named_http(self):
        local_url = "http://http/prefix"
        (hostname, port, prefix) = parse_server_info(local_url)
        self.assertEqual("http", hostname)
        self.assertEqual(DEFAULT_PORT, port)
        self.assertEqual('/prefix', prefix)

    def test_one_slash_port_prefix(self):
        local_url = "https/bogaddy:80/candlepin"
        self.assertRaises(ServerUrlParseErrorScheme,
                          parse_server_info,
                          local_url)

    def test_host_named_http_port_prefix(self):
        local_url = "https://https:8000/prefix"
        (hostname, port, prefix) = parse_server_info(local_url)
        self.assertEqual("https", hostname)
        self.assertEqual("8000", port)
        self.assertEqual('/prefix', prefix)

    def test_host_name_non_numeric_port(self):
        local_url = "https://example.com:https/prefix"
        self.assertRaises(ServerUrlParseErrorPort,
                          parse_server_info,
                          local_url)


# TestParseServerInfo pretty much covers this code wise
class TestParseBaseUrlInfo(fixture.SubManFixture):
    def test_hostname_with_scheme(self):
        # this is the default, so test it here
        local_url = "https://cdn.redhat.com"
        (hostname, port, prefix) = parse_baseurl_info(local_url)
        self.assertEqual(DEFAULT_CDN_HOSTNAME, hostname)
        self.assertEqual(DEFAULT_CDN_PORT, port)
        self.assertEqual("/", prefix)

    def test_format_base_url(self):
        local_url = "https://cdn.redhat.com"
        (hostname, port, prefix) = parse_baseurl_info(local_url)
        self.assertEqual(local_url, format_baseurl(hostname, port, prefix))

    def test_format_base_url_with_port(self):
        local_url = "https://cdn.redhat.com:443"
        (hostname, port, prefix) = parse_baseurl_info(local_url)
        self.assertEqual(prefix, DEFAULT_CDN_PREFIX)
        self.assertEqual("https://cdn.redhat.com", format_baseurl(hostname, port, prefix))

    def test_format_thumbslug_url_with_port(self):
        local_url = "https://someserver.example.com:8088"
        (hostname, port, prefix) = parse_baseurl_info(local_url)
        self.assertEqual(prefix, DEFAULT_CDN_PREFIX)
        self.assertEqual("https://someserver.example.com:8088", format_baseurl(hostname, port, prefix))

    def test_format_not_fqdn_with_port(self):
        local_url = "https://foo-bar:8088"
        (hostname, port, prefix) = parse_baseurl_info(local_url)
        self.assertEqual(prefix, DEFAULT_CDN_PREFIX)
        self.assertEqual("https://foo-bar:8088", format_baseurl(hostname, port, prefix))


class TestUrlBaseJoinEmptyBase(fixture.SubManFixture):

    def test_blank_base_blank_url(self):
        self.assertEqual("",
                          url_base_join("", ""))

    def test_blank_base_url(self):
        url = "http://foo.notreal/"
        self.assertEqual(url,
                          url_base_join("", url))

    def test_blank_base_url_fragment(self):
        url = "baz"
        self.assertEqual(url,
                          url_base_join("", url))

# Not sure this makes sense.
#    def test_blank_base_url_fragment_slash(self):
#        url = "/baz"
#        self.assertEquals(url,
#                          url_base_join("", url))


class TestUrlBaseJoin(fixture.SubManFixture):
    base = "http://foo/bar"

    def test_file_url(self):
        # File urls should be preserved
        self.assertEqual("file://this/is/a/file",
            url_base_join(self.base, "file://this/is/a/file"))

    def test_http_url(self):
        # Http locations should be preserved
        self.assertEqual("http://this/is/a/url",
            url_base_join(self.base, "http://this/is/a/url"))

    def test_blank_url(self):
        # Blank should remain blank
        self.assertEqual("",
            url_base_join(self.base, ""))

    def test_url_fragments(self):
        # Url Fragments should work
        self.assertEqual(self.base + "/baz",
            url_base_join(self.base, "baz"))
        self.assertEqual(self.base + "/baz",
            url_base_join(self.base, "/baz"))

    def test_base_slash(self):
        base = self.base + '/'
        self.assertEqual(self.base + "/baz",
            url_base_join(base, "baz"))
        self.assertEqual(self.base + "/baz",
            url_base_join(base, "/baz"))


class TestUrlBaseJoinFileUrl(TestUrlBaseJoin):
    base = "file:///etc"


class TestUrlBaseJoinJustHttpScheme(TestUrlBaseJoin):
    base = "http://"


class TestUrlBaseJoinHttps(TestUrlBaseJoin):
    base = "https://somesecure.cdn.example.notreal"


class TestUrlBaseJoinHostname(TestUrlBaseJoin):
    base = "cdn.redhat.com"


class TestUrlBaseJoinDefaultCdn(TestUrlBaseJoin):
    base = "https://cdn.redhat.com"


NOT_COLLECTED = "non-collected-package"


class TestGetServerVersions(fixture.SubManFixture):

    def setUp(self):
        get_supported_resources_patcher = patch('subscription_manager.utils.get_supported_resources')
        self.mock_get_resources = get_supported_resources_patcher.start()
        self.mock_get_resources.return_value = ['status']
        self.addCleanup(self.mock_get_resources.stop)

    @patch('subscription_manager.utils.ClassicCheck')
    def test_get_server_versions_classic(self, MockClassicCheck):
        self._inject_mock_invalid_consumer()
        instance = MockClassicCheck.return_value
        instance.is_registered_with_classic.return_value = True

        sv = get_server_versions(None)
        self.assertEqual(sv['server-type'], "RHN Classic")
        self.assertEqual(sv['candlepin'], "Unknown")

    @patch('rhsm.connection.UEPConnection')
    @patch('subscription_manager.utils.ClassicCheck')
    def test_get_server_versions_cp_no_status(self, mock_classic, MockUep):
        instance = mock_classic.return_value
        instance.is_registered_with_classic.return_value = False
        self._inject_mock_valid_consumer()
        MockUep.supports_resource.return_value = False
        sv = get_server_versions(MockUep)
        self.assertEqual(sv['server-type'], 'Red Hat Subscription Management')
        self.assertEqual(sv['candlepin'], "Unknown")

    @patch('rhsm.connection.UEPConnection')
    @patch('subscription_manager.utils.ClassicCheck')
    def test_get_server_versions_cp_with_status(self, mock_classic, MockUep):
        instance = mock_classic.return_value
        instance.is_registered_with_classic.return_value = False
        self._inject_mock_valid_consumer()
        MockUep.supports_resource.return_value = True
        MockUep.getStatus.return_value = {'version': '101', 'release': '23423c', 'rulesVersion': '6.1'}
        sv = get_server_versions(MockUep)
        self.assertEqual(sv['server-type'], 'Red Hat Subscription Management')
        self.assertEqual(sv['candlepin'], '101-23423c')
        self.assertEqual(sv['rules-version'], '6.1')

    @patch('rhsm.connection.UEPConnection')
    @patch('subscription_manager.utils.ClassicCheck')
    def test_get_server_versions_cp_with_status_no_rules_version(self, mock_classic, MockUep):
        instance = mock_classic.return_value
        instance.is_registered_with_classic.return_value = False
        self._inject_mock_valid_consumer()
        MockUep.supports_resource.return_value = True
        MockUep.getStatus.return_value = {'version': '101', 'release': '23423c'}
        sv = get_server_versions(MockUep)
        self.assertEqual(sv['server-type'], 'Red Hat Subscription Management')
        self.assertEqual(sv['candlepin'], '101-23423c')
        self.assertEqual(sv['rules-version'], 'Unknown')

    @patch('rhsm.connection.UEPConnection')
    @patch('subscription_manager.utils.ClassicCheck')
    def test_get_server_versions_cp_with_status_no_keys(self, mock_classic, MockUep):
        instance = mock_classic.return_value
        instance.is_registered_with_classic.return_value = False
        self._inject_mock_valid_consumer()
        MockUep.supports_resource.return_value = True
        MockUep.getStatus.return_value = {}
        sv = get_server_versions(MockUep)
        self.assertEqual(sv['server-type'], 'Red Hat Subscription Management')
        self.assertEqual(sv['candlepin'], 'Unknown-Unknown')
        self.assertEqual(sv['rules-version'], 'Unknown')

    @patch('rhsm.connection.UEPConnection')
    @patch('subscription_manager.utils.ClassicCheck')
    def test_get_server_versions_cp_with_status_bad_data(self, mock_classic, MockUep):
        instance = mock_classic.return_value
        instance.is_registered_with_classic.return_value = False
        self._inject_mock_valid_consumer()
        MockUep.supports_resource.return_value = True

        dataset = [
            {'version': None, 'release': '123'},
            {'version': 123, 'release': '123'},

            {'version': '123', 'release': None},
            {'version': '123', 'release': 123},

            {'version': None, 'release': None},
            {'version': None, 'release': 123},
            {'version': 123, 'release': None},
            {'version': 123, 'release': 123},
        ]

        for value in dataset:
            MockUep.getStatus.return_value = value
            sv = get_server_versions(MockUep)
            self.assertEqual(sv['server-type'], 'Red Hat Subscription Management')
            self.assertEqual(sv['candlepin'], 'Unknown')
            self.assertEqual(sv['rules-version'], 'Unknown')

    @patch('rhsm.connection.UEPConnection')
    @patch('subscription_manager.utils.ClassicCheck')
    def test_get_server_versions_cp_with_status_and_classic(self, mock_classic, MockUep):
        instance = mock_classic.return_value
        instance.is_registered_with_classic.return_value = True
        self._inject_mock_valid_consumer()
        MockUep.supports_resource.return_value = True
        MockUep.getStatus.return_value = {'version': '101', 'release': '23423c', 'rulesVersion': '6.1'}
        sv = get_server_versions(MockUep)
        self.assertEqual(sv['server-type'], 'RHN Classic and Red Hat Subscription Management')
        self.assertEqual(sv['candlepin'], '101-23423c')
        self.assertEqual(sv['rules-version'], '6.1')

    @patch('rhsm.connection.UEPConnection')
    @patch('subscription_manager.utils.ClassicCheck')
    def test_get_server_versions_cp_exception(self, mock_classic, MockUep):
        def raise_exception(arg):
            raise Exception("boom")
        instance = mock_classic.return_value
        instance.is_registered_with_classic.return_value = False
        self._inject_mock_valid_consumer()
        self.mock_get_resources.side_effect = raise_exception
        MockUep.getStatus.return_value = {'version': '101', 'release': '23423c'}
        sv = get_server_versions(MockUep)
        print(sv)
        self.assertEqual(sv['server-type'], "Red Hat Subscription Management")
        self.assertEqual(sv['candlepin'], "Unknown")

    @patch('rhsm.connection.UEPConnection')
    @patch('subscription_manager.utils.ClassicCheck')
    def test_get_server_versions_cp_exception_and_classic(self, mock_classic, MockUep):
        def raise_exception(arg):
            raise Exception("boom")
        instance = mock_classic.return_value
        instance.is_registered_with_classic.return_value = True
        self._inject_mock_invalid_consumer()
        self.mock_get_resources.side_effect = raise_exception
        MockUep.getStatus.return_value = {'version': '101', 'release': '23423c'}
        sv = get_server_versions(MockUep)
        self.assertEqual(sv['server-type'], "RHN Classic")
        self.assertEqual(sv['candlepin'], "Unknown")


class TestGetClientVersions(fixture.SubManFixture):
    @patch('subscription_manager.utils.subscription_manager.version')
    def test_get_client_versions(self, mock_sub_version):
        mock_sub_version.rpm_version = '9.8.7-6'
        cv = get_client_versions()
        self.assertEqual(cv['subscription-manager'], "9.8.7-6")
        self.assertTrue(isinstance(cv['subscription-manager'], str))

    @patch('subscription_manager.utils.subscription_manager.version')
    def test_get_client_versions_strings(self, mock_sub_version):
        mock_sub_version.rpm_version = 'ef-gh'
        cv = get_client_versions()
        self.assertEqual(cv['subscription-manager'], "ef-gh")


class TestGetVersion(fixture.SubManFixture):
    def test_version_and_release_present(self):
        versions = Mock()
        versions.get_version.return_value = "1.0"
        versions.get_release.return_value = "1"
        result = get_version(versions, "foobar")
        self.assertEqual("1.0-1", result)

    def test_version_no_release(self):
        versions = Mock()
        versions.get_version.return_value = "1.0"
        versions.get_release.return_value = ""
        result = get_version(versions, "foobar")
        self.assertEqual("1.0", result)


class TestFriendlyJoin(fixture.SubManFixture):

    def test_join(self):
        self.assertEqual("One", friendly_join(["One"]))
        self.assertEqual("One and Two", friendly_join(["One", "Two"]))
        self.assertEqual("One, Two, and Three", friendly_join(["One", "Two", "Three"]))
        self.assertEqual("", friendly_join([]))
        self.assertEqual("", friendly_join(None))
        self.assertEqual("", friendly_join(set()))
        self.assertEqual("One", friendly_join([None, "One"]))
        self.assertEqual("One", friendly_join(["One", None]))
        self.assertEqual("", friendly_join([None, None, None]))

        # We allow any iterable, so test a set created from a list with dupes
        words = set(["Two", "One", "Two", "Two"])
        res = friendly_join(words)
        self.assertTrue(res in ["One and Two", "Two and One"])

        self.assertEqual("1, 2, 3, 4, 5, 6, and fish",
                          friendly_join([1, 2, u"3", 4, "5", 6, "fish"]))


class TestTrueValue(fixture.SubManFixture):

    def test_true_value(self):
        self.assertTrue(is_true_value("1"))
        self.assertTrue(is_true_value("True"))
        self.assertTrue(is_true_value("true"))
        self.assertTrue(is_true_value("truE"))
        self.assertTrue(is_true_value("yes"))
        self.assertTrue(is_true_value("YeS"))

    def test_false_value(self):
        self.assertFalse(is_true_value("0"))
        self.assertFalse(is_true_value("False"))
        self.assertFalse(is_true_value("FalsE"))
        self.assertFalse(is_true_value("no"))
        self.assertFalse(is_true_value("nO"))
        self.assertFalse(is_true_value("y"))
        self.assertFalse(is_true_value("n"))
        self.assertFalse(is_true_value("t"))
        self.assertFalse(is_true_value("f"))


class TestUniqueListItems(fixture.SubManFixture):
    def test_preserves_order(self):
        input_list = [1, 1, 2, 2, 3, 3]
        expected = [1, 2, 3]
        self.assertEqual(expected, unique_list_items(input_list))

    def test_hash_function(self):
        mock_item_1 = Mock()
        mock_item_1.value = 1
        mock_item_2 = Mock()
        mock_item_2.value = 2
        input_list = [mock_item_1, mock_item_1, mock_item_2, mock_item_2]
        expected = [mock_item_1, mock_item_2]
        self.assertEqual(expected, unique_list_items(input_list, lambda x: x.value))


class TestProductCertificateFilter(fixture.SubManFixture):

    def test_set_filter_string(self):
        test_data = [
            (None, True),

            ("", True),
            ("test", True),
            ("?", True),
            ("*", True),
            ("\\?", True),
            ("\\*", True),
            ("\\\\", True),
            ("*test*", True),
            ("*test\\?", True),
            ("a?b\\\\*c\\?d", True),
            ("**", True),
            ("??", True),
            ("*?", True),
            ("?*", True),
            ("te**st", True),
            ("te*?st", True),
            ("te?*st", True),
            ("te??st", True),
            ("*te\\st*", True),
            ("*te\\\\st*", True),
            ("*te\*\?st*", True),

            (123, False),
            (True, False),
            (False, False),
            (["nope"], False),
            ({"key": "value"}, False)
        ]

        for (index, data) in enumerate(test_data):
            cert_filter = ProductCertificateFilter()
            result = cert_filter.set_filter_string(data[0])

            self.assertEqual(result, data[1], "ProductCertificateFilter.set_filter_string failed with data set %i.\nActual:   %s\nExpected: %s)" % (index, result, data[1]))

    def test_match(self):
        prod_cert = StubProductCertificate(product=StubProduct(name="test*product?", product_id="123456789"))

        test_data = [
            (None, prod_cert, False),
            ("", prod_cert, False),
            ("test", prod_cert, False),
            ("product", prod_cert, False),
            ("test\*product\?", prod_cert, True),
            ("test*", prod_cert, True),
            ("test*********", prod_cert, True),
            ("test?????????", prod_cert, True),
            ("*product?", prod_cert, True),
            ("*product\?", prod_cert, True),
            ("?????product*", prod_cert, True),
            ("*****product*", prod_cert, True),
            ("?????????????", prod_cert, True),
            ("*************", prod_cert, True),
            ("??**??*??**??", prod_cert, True),
            ("**??**?**??**", prod_cert, True),
            ("test?product\?", prod_cert, True),
            ("test*product?", prod_cert, True),
            ("test*nope", prod_cert, False),
            ("nope*product", prod_cert, False),
            ("*nope*", prod_cert, False),
            ("test*nope*product", prod_cert, False),

            ("1234*", prod_cert, True),
            ("*4567*", prod_cert, True),
            ("123???789", prod_cert, True),
            ("??34567??", prod_cert, True),
            ("*4?6*", prod_cert, True),
            ("1234", prod_cert, False),
            ("??123456789", prod_cert, False),
        ]

        for (index, data) in enumerate(test_data):
            cert_filter = ProductCertificateFilter(filter_string=data[0])
            result = cert_filter.match(data[1])

            self.assertEqual(result, data[2], "ProductCertificateFilter.match failed with data set %i.\nActual:   %s\nExpected: %s" % (index, result, data[2]))


class TestEntitlementCertificateFilter(fixture.SubManFixture):

    def test_set_service_level(self):
        test_data = [
            (None, True),
            ("", True),
            ("Bacon", True),
            ("Cheese", True),
            ("Burger", True),

            (123, False),
            (True, False),
            (False, False),
            (["nope"], False),
            ({"key": "value"}, False)
        ]

        for (index, data) in enumerate(test_data):
            cert_filter = EntitlementCertificateFilter()
            result = cert_filter.set_service_level(data[0])

            self.assertEqual(result, data[1], "EntitlementCertificateFilter.set_service_level failed with data set %i.\nActual:   %s\nExpected: %s)" % (index, result, data[1]))

    def test_match(self):
        ent_cert = StubEntitlementCertificate(product=StubProduct(name="test*entitlement?", product_id="123456789"), service_level="Premium", provided_products=[
                "test product b",
                "beta product 1",
                "shared product",
                "back\\slash"
        ])
        # Order information is hard-coded in the stub, so we've to modify it
        # separately:
        ent_cert.order.contract = "Contract-A"

        no_sla_ent_cert = StubEntitlementCertificate(
            product=StubProduct(name="nobodycares", product_id="123456789"),
            service_level=None, provided_products=[])

        test_data = [
            (None, None, ent_cert, False),
            ("*entitlement*", None, ent_cert, True),
            ("*shared*", None, ent_cert, True),
            ("beta*", None, ent_cert, True),
            ("123456789", None, ent_cert, True),
            ("prem*", None, ent_cert, True),                   # service level via --contains-text vs --servicelevel
            ("*contract*", None, ent_cert, True),
            ("contract-a", None, ent_cert, True),
            ("contract-b", None, ent_cert, False),
            ("*entitlement*", "Premium", ent_cert, True),
            ("*shared*", "Premium", ent_cert, True),
            ("beta*", "Premium", ent_cert, True),
            ("123456789", "Premium", ent_cert, True),
            ("prem*", "Premium", ent_cert, True),              # ^
            ("*contract*", "Premium", ent_cert, True),
            ("contract-a", "Premium", ent_cert, True),
            ("contract-b", "Premium", ent_cert, False),
            ("*entitlement*", "Standard", ent_cert, False),
            ("*shared*", "Standard", ent_cert, False),
            ("beta*", "Standard", ent_cert, False),
            ("123456789", "Standard", ent_cert, False),
            ("prem*", "Standard", ent_cert, False),            # ^
            ("*contract*", "Standard", ent_cert, False),
            ("contract-a", "Standard", ent_cert, False),
            ("contract-b", "Standard", ent_cert, False),
            (None, "Premium", ent_cert, True),
            (None, "pReMiUm", ent_cert, True),
            (None, "Standard", ent_cert, False),
            (None, "sTANDard", ent_cert, False),
            (None, "aslfk;", ent_cert, False),
            (None, "", ent_cert, False),
            (None, "", no_sla_ent_cert, True),
        ]

        for (index, data) in enumerate(test_data):
            cert_filter = EntitlementCertificateFilter(
                filter_string=data[0], service_level=data[1])
            result = cert_filter.match(data[2])

            self.assertEqual(result, data[3], "EntitlementCertificateFilter.match failed with data set %i.\nActual:   %s\nExpected: %s" % (index, result, data[3]))


class TestIsOwnerUsingSimpleContentAccess(fixture.SubManFixture):
    """
    Class used for testing function is_simple_content_access
    """

    MOCK_ENTITLEMENT_OWNER = {"contentAccessMode": "entitlement"}
    MOCK_ORG_ENVIRONMENT_OWNER = {"contentAccessMode": "org_environment"}

    def setUp(self):
        super(TestIsOwnerUsingSimpleContentAccess, self).setUp()
        self.cp_provider = Mock()
        self.mock_uep = Mock()
        self.mock_uep.getOwner = Mock(return_value=self.MOCK_ENTITLEMENT_OWNER)
        self.cp_provider.get_consumer_auth_cp = Mock(return_value=self.mock_uep)
        self.identity = Mock()
        self.identity.uuid = Mock(return_value="7f85da06-5c35-44ba-931d-f11f6e581f89")

    def test_get_entitlement_owner(self):
        ret = is_simple_content_access(uep=self.mock_uep, identity=self.identity)
        self.assertFalse(ret)

    def test_get_org_environment_owner(self):
        self.mock_uep.getOwner = Mock(return_value=self.MOCK_ORG_ENVIRONMENT_OWNER)
        ret = is_simple_content_access(uep=self.mock_uep, identity=self.identity)
        self.assertTrue(ret)


PROCESS_STATUS_FILE = \
"""Umask:	0000
State:	S (sleeping)
Tgid:	1
Ngid:	0
Pid:	1
PPid:	0
TracerPid:	0
Uid:	0	0	0	0
Gid:	0	0	0	0
FDSize:	512
Groups:
NStgid:	1
NSpid:	1
NSpgid:	1
NSsid:	1
VmPeak:	  237396 kB
VmSize:	  180056 kB
VmLck:	       0 kB
VmPin:	       0 kB
VmHWM:	   16216 kB
VmRSS:	    7800 kB
RssAnon:	    3768 kB
RssFile:	    4032 kB
RssShmem:	       0 kB
VmData:	   29584 kB
VmStk:	     132 kB
VmExe:	     912 kB
VmLib:	    9380 kB
VmPTE:	     108 kB
VmSwap:	    1564 kB
HugetlbPages:	       0 kB
CoreDumping:	0
THP_enabled:	1
Threads:	1
SigQ:	1/63421
SigPnd:	0000000000000000
ShdPnd:	0000000000000000
SigBlk:	7be3c0fe28014a03
SigIgn:	0000000000001000
SigCgt:	00000001800004ec
CapInh:	0000000000000000
CapPrm:	0000003fffffffff
CapEff:	0000003fffffffff
CapBnd:	0000003fffffffff
CapAmb:	0000000000000000
NoNewPrivs:	0
Seccomp:	0
Speculation_Store_Bypass:	thread vulnerable
Cpus_allowed:	f
Cpus_allowed_list:	0-3
Mems_allowed:	00000000,00000000,00000000,00000000,00000000,00000000,00000000,00000000,00000000,00000000,00000000,00000000,00000000,00000000,00000000,00000000,00000000,00000000,00000000,00000000,00000000,00000000,00000000,00000000,00000000,00000000,00000000,00000000,00000000,00000000,00000000,00000001
Mems_allowed_list:	0
voluntary_ctxt_switches:	4002321
nonvoluntary_ctxt_switches:	16572
"""

PROCESS_STATUS_FILE_LINES = PROCESS_STATUS_FILE.split('\n')

original_open = open


class TestGetProcessNamesAndIsProcessRunning(fixture.SubManFixture):
    """
    Class used for testing get_process_names and is_process_running functions
    """

    def setUp(self):
        super(TestGetProcessNamesAndIsProcessRunning, self).setUp()
        self.root_dir = mkdtemp()
        self.proc_root = os.path.join(self.root_dir, 'proc')
        os.mkdir(self.proc_root)
        # self.addCleanup(rmtree, self.root_dir)
        self.fake_pids = set()

    def create_fake_process_status(self, name=None, randomize_name_location=False):
        """
        Given a name of a fake process, create a directory representing
        the kernel-provided info relevant to our use for that fake process.
        """
        # The kernel gets to decide the actual largest PID
        # but for our purposes it just needs to be a number
        fake_pid = random.randint(0, 123456789)
        while fake_pid in self.fake_pids:
            fake_pid = random.randint(0, 123456789)

        pid_dir = os.path.join(self.proc_root, str(fake_pid))
        os.mkdir(pid_dir)

        process_status_contents = PROCESS_STATUS_FILE_LINES
        if name:
            if randomize_name_location:
                location = random.randint(1, len(process_status_contents) - 1)
            else:
                location = 0
            process_status_contents.insert(location, "Name:\t{name}")

        with open(os.path.join(pid_dir, 'status'), 'w') as status:
            process_status_contents = "\n".join(process_status_contents)
            status.write(process_status_contents.format(name=name))

    @staticmethod
    def redirect_open(new_root):
        """
        Construct a suitable side_effect for a mock of the open builtin
        function, such that any path that is opened is transformed to
        {new_root}/{original_path}
        """
        def new_open(*args, **kwargs):
            original_path = args[0]
            if original_path.startswith(os.path.sep):
                original_path = original_path[1:]
            corrected_path = os.path.join(new_root, original_path)
            print(corrected_path)
            return original_open(corrected_path, *args[1:], **kwargs)
        return new_open

    def run_with_mock_proc(self, func, *args, **kwargs):
        """
        Mock out access to various os functions and to open from the same
        mock root directory. Should we need to expand our fake chroot,
        this would be the place to do it
        """
        mock_proc_dirs = os.listdir(self.proc_root)
        new_open = self.redirect_open(self.root_dir)
        os_patch = patch('subscription_manager.utils.os.listdir')
        m = os_patch.start()
        m.return_value = mock_proc_dirs
        open_patch = patch(fixture.OPEN_FUNCTION)
        mopen = open_patch.start()
        mopen.side_effect = new_open
        try:
            res = func(*args, **kwargs)
        finally:
            open_patch.stop()
            os_patch.stop()
        return res

    def test_is_process_running(self):
        """
        Test if process is running
        """
        made_up_process_name = "magic_process"
        res = self.run_with_mock_proc(is_process_running, made_up_process_name)
        self.assertFalse(res)

        self.create_fake_process_status(made_up_process_name)
        # Now we should be able to read that magic_process's status

        res = self.run_with_mock_proc(is_process_running, made_up_process_name)
        self.assertTrue(res)

    def test_is_process_running_missing_name(self):
        """
        Test if the status file we are reading from /proc has no "Name: thing" like line.
        """
        made_up_process_name = "magic_process"
        res = self.run_with_mock_proc(is_process_running, made_up_process_name)
        self.assertFalse(res)

        self.create_fake_process_status(name=None)

        res = self.run_with_mock_proc(is_process_running, made_up_process_name)
        self.assertFalse(res)

    def test_is_process_running_weird_name_location(self):
        """
        Test if process is running when the "Name: thing" line is not the first line
        """
        made_up_process_name = "magic_process"
        res = self.run_with_mock_proc(is_process_running, made_up_process_name)
        self.assertFalse(res)

        self.create_fake_process_status(name=made_up_process_name, randomize_name_location=True)

        res = self.run_with_mock_proc(is_process_running, made_up_process_name)
        self.assertTrue(res)

    def test_get_process_names(self):
        """
        Test getting list of processes
        """
        fake_process_name = "magic_process"
        self.create_fake_process_status(name=fake_process_name)
        ld = os.listdir(self.proc_root)
        new_open = self.redirect_open(self.root_dir)
        os_patch = patch('subscription_manager.utils.os.listdir')
        m = os_patch.start()
        m.return_value = ld
        open_patch = patch(fixture.OPEN_FUNCTION)
        mopen = open_patch.start()
        mopen.side_effect = new_open
        res = get_process_names()
        res = list(res)
        self.assertEquals(res, [fake_process_name], "Expected an empty list, Actual: %s" % res)



from __future__ import print_function, division, absolute_import

# Copyright (c) 2011 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.
#
try:
    import unittest2 as unittest
except ImportError:
    import unittest

from .stubs import StubEntitlementCertificate, StubProduct
from mock import Mock
from subscription_manager.reasons import Reasons

INST_PID_1 = "100000000000002"  # awesomeos 64
ENT_ID_1 = "ff8080813e468fd8013e4690966601d7"
INST_PID_2 = "100000000000003"  # ppc64 awesomeos
ENT_ID_2 = "ff8080813e468fd8013e4694a4921179"
INST_PID_3 = "801"  # non-entitled ram limiting product
INST_PID_4 = "900"  # multiattr stack
ENT_ID_4 = "ff8080813e468fd8013e4690f041031b"

# not in valid ents
NOT_VALID_ENT_ID = "ff8080813e468fd802344690f041031b"

STACK_1 = 'multiattr-stack-test'  # multiattr
STACK_2 = '1'  # awesomeos 64

PARTIAL_STACK_ID = STACK_1
PROD_4 = StubProduct(INST_PID_4,
        name="Multi-Attribute Stackable")
PROD_2 = StubProduct(INST_PID_2,
        name="Awesome OS for ppc64")
PROD_1 = StubProduct(INST_PID_1,
        name="Awesome OS for x86_64")


class ReasonsTests(unittest.TestCase):

    def setUp(self):
        self.sorter = Mock()
        self.sorter.valid_products = [INST_PID_1]
        self.sorter.valid_entitlement_certs = [StubEntitlementCertificate(PROD_2, ent_id=ENT_ID_2),
                StubEntitlementCertificate(PROD_1, ent_id=ENT_ID_1),
                StubEntitlementCertificate(product=PROD_4, stacking_id=STACK_1,
                    ent_id=ENT_ID_4),
                StubEntitlementCertificate(StubProduct('not_installed_product',
                    name="Some Product"),
                    ent_id="SomeSubId")]
        reason_list = []
        reason_list.append(self.build_reason('NOTCOVERED',
            'Not covered by a valid subscription.',
            {'product_id': '801',
             'name': 'RAM Limiting Product'}))
        reason_list.append(self.build_ent_reason_with_attrs('CORES',
            'Only covers 16 of 32 cores.',
            '32', '16', name='Multi-Attribute Stackable (16 cores, 4 sockets, 8GB RAM)',
            stack='multiattr-stack-test'))
        reason_list.append(self.build_ent_reason_with_attrs('SOCKETS',
            'Only covers 4 of 8 sockets.',
            '8', '4', name='Multi-Attribute Stackable (16 cores, 4 sockets, 8GB RAM)',
            stack='multiattr-stack-test'))
        reason_list.append(self.build_ent_reason_with_attrs('RAM',
            'Only covers 8GB of 31GB of RAM.',
            '31', '8', name='Multi-Attribute Stackable (16 cores, 4 sockets, 8GB RAM)',
            stack='multiattr-stack-test'))
        reason_list.append(self.build_ent_reason_with_attrs('RAM',
            'A different way to say Only covers 8GB of 31GB of RAM.',
            '31', '8', name='Multi-Attribute Stackable (16 cores, 4 sockets, 8GB RAM)',
            stack='multiattr-stack-test'))
        # This is a dupe of the above, since reasons.py has code for detecting
        # dupe messages or names.
        reason_list.append(self.build_ent_reason_with_attrs('RAM',
            'Only covers 8GB of 31GB of RAM.',
            '31', '8', name='Multi-Attribute Stackable (16 cores, 4 sockets, 8GB RAM)',
            stack='multiattr-stack-test'))
        reason_list.append(self.build_ent_reason_with_attrs('ARCH',
            'Covers architecture ppc64 but the system is x86_64.',
            'x86_64', 'ppc64', name='Awesome OS for ppc64',
            ent=ENT_ID_2))

        self.sorter.reasons = Reasons(reason_list, self.sorter)

    def test_get_stack_subscriptions(self):
        subs = self.sorter.reasons.get_stack_subscriptions(PARTIAL_STACK_ID)
        self.assertEqual(1, len(subs))
        self.assertEqual(ENT_ID_4, subs[0])

    def test_get_product_subscriptions(self):
        subs = self.sorter.reasons.get_product_subscriptions(PROD_4)
        self.assertEqual(1, len(subs))
        self.assertEqual(ENT_ID_4, subs[0].subject['CN'])

    def test_get_product_reasons(self):
        messages = self.sorter.reasons.get_product_reasons(PROD_4)
        self.assertEqual(4, len(messages))
        expectations = []
        expectations.append("Only covers 16 of 32 cores.")
        expectations.append("Only covers 8GB of 31GB of RAM.")
        expectations.append("Only covers 4 of 8 sockets.")
        for expected in expectations:
            self.assertTrue(expected in messages)

        messages = self.sorter.reasons.get_product_reasons(PROD_2)
        self.assertEqual(1, len(messages))
        expected = "Covers architecture ppc64 but the system is x86_64."
        self.assertEqual(expected, messages[0])

        reason = self.build_ent_reason_with_attrs('SOCKETS', 'some message', '8', '6',
                prod=INST_PID_1, name="Awesome OS for x86_64")
        self.sorter.reasons.reasons.append(reason)
        messages = self.sorter.reasons.get_product_reasons(PROD_1)
        self.assertEqual(0, len(messages))
        self.sorter.reasons.reasons.remove(reason)

    def test_get_subscription_reasons(self):
        sub_reasons = self.sorter.reasons.get_subscription_reasons(ENT_ID_1)
        self.assertEqual(0, len(sub_reasons))

        sub_reasons = self.sorter.reasons.get_subscription_reasons(ENT_ID_2)
        self.assertEqual(1, len(sub_reasons))
        expected = "Covers architecture ppc64 but the system is x86_64."
        actual = sub_reasons[0]
        self.assertEqual(expected, actual)

    def test_get_subscription_reasons_map(self):
        sub_reason_map = self.sorter.reasons.get_subscription_reasons_map()
        self.assertEqual(4, len(sub_reason_map[ENT_ID_4]))
        self.assertEqual(0, len(sub_reason_map[ENT_ID_1]))
        self.assertEqual(1, len(sub_reason_map[ENT_ID_2]))
        expected = "Covers architecture ppc64 but the system is x86_64."
        actual = sub_reason_map[ENT_ID_2][0]
        self.assertEqual(expected, actual)

    def test_get_subscription_reasons_map_ent_id_not_in_valid_ent_certs(self):
        reason_expired_entid = \
            self.build_ent_reason_with_attrs(key='ARCH', message='This is not a reason reasons.',
                                             has='x86_64', covered='ppc64',
                                             name='Awesome OS for ppc64',
                                             ent=NOT_VALID_ENT_ID)

        self.sorter.valid_entitlement_certs = []
        self.sorter.reasons = Reasons([reason_expired_entid], self.sorter)

        sub_reason_map = self.sorter.reasons.get_subscription_reasons_map()
        self.assertTrue(NOT_VALID_ENT_ID not in sub_reason_map)

    def test_get_subscription_reasons_map_message_not_in_result(self):
        # This is to cover the duplicate message finding code
        reason_expired_entid = \
            self.build_ent_reason_with_attrs(key='ARCH',
                                             message='Some new reason.',
                                             has='x86_64', covered='ppc64',
                                             name='Awesome OS for ppc64',
                                             ent=ENT_ID_2)

        reason_expired_entid_2 = \
            self.build_ent_reason_with_attrs(key='ARCH',
                                             message='Some new reason.',
                                             has='x86_64', covered='ppc64',
                                             name='Awesome OS for ppc64',
                                             ent=ENT_ID_2)
        another_stack_reason = \
        self.build_ent_reason_with_attrs(key='SOCKETS',
                                         message='Only covers 4 of 8 sockets.',
                                         has='8',
                                         covered='4',
                                         name='Multi-Attribute Stackable (16 cores, 4 sockets, 8GB RAM)',
                                         stack='multiattr-stack-test')

        self.sorter.reasons.reasons.append(reason_expired_entid)
        self.sorter.reasons.reasons.append(reason_expired_entid_2)
        self.sorter.reasons.reasons.append(another_stack_reason)

        sub_reason_map = self.sorter.reasons.get_subscription_reasons_map()
        self.assertTrue(ENT_ID_2 in sub_reason_map)

    def test_get_reason_id(self):
        reason = self.build_ent_reason_with_attrs(
                'SOCKETS', 'some message', '8', '6', ent='1234')
        reason_id = self.sorter.reasons.get_reason_id(reason)
        self.assertEqual("Subscription 1234", reason_id)
        reason = self.build_ent_reason_with_attrs(
                'SOCKETS', 'some message', '8', '6', stack='1234')
        reason_id = self.sorter.reasons.get_reason_id(reason)
        self.assertEqual("Stack 1234", reason_id)
        reason = self.build_ent_reason_with_attrs(
                'SOCKETS', 'some message', '8', '6', prod='1234')
        reason_id = self.sorter.reasons.get_reason_id(reason)
        self.assertEqual("Product 1234", reason_id)

    def test_get_name_message_map(self):
        name_message_map = self.sorter.reasons.get_name_message_map()
        self.assertEqual(3, len(name_message_map))
        expected = ['A different way to say Only covers 8GB of 31GB of RAM.',
                    'Only covers 16 of 32 cores.',
                    'Only covers 4 of 8 sockets.',
                    'Only covers 8GB of 31GB of RAM.']
        self.assortEquals(expected, name_message_map[
            'Multi-Attribute Stackable (16 cores, 4 sockets, 8GB RAM)'])

    def set_up_duplicates(self):
        self.sorter.reasons.reasons = []
        self.sorter.reasons.reasons.append(self.build_ent_reason_with_attrs(
            'SOCKETS', 'some message', '8', '6', ent='1234', name='testing'))
        self.sorter.reasons.reasons.append(self.build_ent_reason_with_attrs(
            'SOCKETS', 'some message', '8', '6', ent='2345', name='testing'))
        self.sorter.reasons.reasons.append(self.build_ent_reason_with_attrs(
            'SOCKETS', 'some message', '8', '6', ent='3345', name='testing'))

    def build_ent_reason_with_attrs(self, key, message, has,
            covered, name=None, ent=None, stack=None, prod=None):
        attrs = {'has': has,
                'covered': covered}
        if name:
            attrs['name'] = name
        if ent:
            attrs['entitlement_id'] = ent
        elif stack:
            attrs['stack_id'] = stack
        elif prod:
            attrs['product_id'] = prod
        return self.build_reason(key, message, attrs)

    def build_reason(self, key, message, attrs):
        return {'key': key,
                'message': message,
                'attributes': attrs}

    def assortEquals(self, list_a, list_b):
        self.assertTrue(isinstance(list_a, list) and isinstance(list_b, list))
        self.assertEqual(sorted(list_a), sorted(list_b))



# -*- coding: utf-8 -*-

from __future__ import print_function, division, absolute_import
#
# Copyright (c) 2018 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public License,
# version 2 (GPLv2). There is NO WARRANTY for this software, express or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. You should have received a copy of GPLv2
# along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#
# Red Hat trademarks are not licensed under GPLv2. No permission is
# granted to use or replicate Red Hat trademarks that are incorporated
# in this software or its documentation.

# A group of tests for the miscellaneous utilities in the utils module of syspurpose

from .base import SyspurposeTestBase
import io
import os
import errno
import json
import mock

from syspurpose import utils


class UtilsTests(SyspurposeTestBase):

    def tearDown(self):
        utils.HOST_CONFIG_DIR = "/etc/rhsm-host/"

    @mock.patch('syspurpose.utils.system_exit')
    def test_create_dir(self, mock_system_exit):
        """
        Verify that the create_dir utility method creates directories as we expect.
        :return:
        """
        temp_dir = self._mktmp()

        # A directory that does not exist yet
        new_dir = os.path.join(temp_dir, "new_dir")
        res = self.assertRaisesNothing(utils.create_dir, new_dir)
        self.assertTrue(os.path.exists(new_dir))
        # There was a change, so the result should be True
        self.assertTrue(res)

        # Create another directory
        existing_dir = os.path.join(temp_dir, "existing")
        os.mkdir(existing_dir, 0o644)
        res = self.assertRaisesNothing(utils.create_dir, existing_dir)
        # Should have been no change, so the result should be false
        self.assertFalse(res)

        # Create one more directory that does not have the right permissions
        bad_perm_dir = os.path.join(temp_dir, "bad_perm_dir")
        os.mkdir(bad_perm_dir, 0o400)

        impossible_sub_dir = os.path.join(bad_perm_dir, "any_sub_dir")

        self.assertRaisesNothing(utils.create_dir, impossible_sub_dir)
        self.assertFalse(os.path.exists(impossible_sub_dir))
        # Should try to exit when the directory could not be created due to bad permissions
        mock_system_exit.assert_called_once()

    @mock.patch('syspurpose.utils.system_exit')
    def test_create_file(self, mock_system_exit):
        temp_dir = self._mktmp()
        to_create = os.path.join(temp_dir, "my_cool_file.json")

        test_data = {"arbitrary_key": "arbitrary_value"}

        res = self.assertRaisesNothing(utils.create_file, to_create, test_data)
        self.assertTrue(res)
        self.assertTrue(os.path.exists(to_create))

        with io.open(to_create, 'r', encoding='utf-8') as fp:
            actual_contents = json.load(fp, encoding='utf-8')

        self.assertDictEqual(actual_contents, test_data)

        to_create = os.path.join(temp_dir, "my_super_chill_file.json")

        # And now when the file appears to exist
        with mock.patch('syspurpose.utils.io.open') as mock_open:
            error_to_raise = OSError()
            error_to_raise.errno = errno.EEXIST
            mock_open.side_effect = error_to_raise

            res = self.assertRaisesNothing(utils.create_file, to_create, test_data)
            self.assertFalse(res)

        to_create = os.path.join(temp_dir, "my_other_cool_file.json")

        # And now with an unexpected OSError
        with mock.patch('syspurpose.utils.io.open') as mock_open:
            error_to_raise = OSError()
            error_to_raise.errno = errno.E2BIG  # Anything aside from the ones expected
            mock_open.side_effect = error_to_raise

            self.assertRaises(OSError, utils.create_file, to_create, test_data)
            self.assertFalse(os.path.exists(to_create))

    def test_in_container_host_config_dir_exists(self):
        """
        Verify that in_container returns true when utils.HOST_CONFIG_DIR dir exists.
        :return:
        """
        temp_dir = self._mktmp()
        utils.HOST_CONFIG_DIR = temp_dir
        self.assertTrue(utils.in_container())

    def test_in_container_host_config_dir_does_not_exist(self):
        """
        Verify that in_container returns false when utils.HOST_CONFIG_DIR dir does not exist.
        :return:
        """
        utils.HOST_CONFIG_DIR = "/does/not/exist/"
        self.assertFalse(utils.in_container())
